@@ -61,6 +61,9 @@ KwaveApp::KwaveApp()
     // load the list of plugins
     PluginManager::findPlugins();
 
+    // close when the last window closed
+    connect(this, SIGNAL(lastWindowClosed()), this, SLOT(quit()));
+
 #ifndef UNIQUE_APP
     newInstance();
 #endif
@@ -78,10 +81,9 @@ int KwaveApp::newInstance()
     } else {
 	// open a window for each file specified in the
 	// command line an load it
-	QString filename;
 	for (unsigned int i = 0; i < argc; i++) {
-	    filename = QFile::decodeName(args->arg(i));
-	    newWindow(filename);
+	    QFileInfo file(args->arg(i));
+	    newWindow(file.absFilePath());
 	}
     }
     if (args) args->clear();
@@ -114,7 +116,6 @@ bool KwaveApp::executeCommand(const QString &command)
 //***************************************************************************
 void KwaveApp::addRecentFile(const QString &newfile)
 {
-    ASSERT(newfile.length() != 0);
     if (!newfile.length()) return;
 
     // remove old entries if present
@@ -135,7 +136,7 @@ void KwaveApp::addRecentFile(const QString &newfile)
 }
 
 //***************************************************************************
-bool KwaveApp::newWindow(const QString &filename)
+bool KwaveApp::newWindow(const KURL &url)
 {
     TopWidget *new_top_widget = new TopWidget(*this);
     ASSERT(new_top_widget);
@@ -166,7 +167,7 @@ bool KwaveApp::newWindow(const QString &filename)
     connect(this, SIGNAL(recentFilesChanged()),
             new_top_widget, SLOT(updateRecentFiles()));
 
-    if (filename.length()) new_top_widget->loadFile(filename);
+    if (!url.isEmpty()) new_top_widget->loadFile(url);
 
     return true;
 }
