@@ -16,6 +16,7 @@
  ***************************************************************************/
 
 #include <dlfcn.h>
+#include <errno.h>
 #include <pthread.h>
 
 #include "qobject.h"
@@ -56,6 +57,39 @@
 //****************************************************************************
 //****************************************************************************
 
+//class PluginGarbageCollector: public Thread
+//{
+//public:
+//    virtual void run();
+//
+//slots:
+//    void remove(KwavePlugin *plugin);
+//
+//private:
+//    QQueue<Plugin> m_queue;
+//    SignalProxy1<KwavePlugin *> m_spx_plugin;
+//}
+//
+//void PluginGarbageCollector::run()
+//{
+//    KwavePlugin *plugin;
+//    do {
+//	plugin = m_spx_plugin.dequeue();
+//	if (!plugin) {
+//	    // dequeued a null plugin -> signal to shut down
+//	    debug("PluginGarbageCollector: shutting down...");
+//	    break;
+//	}
+//	
+//	// wait until the plugin's thread is down
+//	
+//	// delete the plugin
+//	
+//	// remove it's handle
+//	
+//    } while (plugin);
+//}
+
 // static initializer
 QMap<QString, QString> PluginManager::m_plugin_files;
 
@@ -243,18 +277,19 @@ KwavePlugin *PluginManager::loadPlugin(const QString &name)
 }
 
 //***************************************************************************
-void PluginManager::executePlugin(const QString &name, QStringList *params)
+int PluginManager::executePlugin(const QString &name, QStringList *params)
 {
     QString command;
+    int result = 0;
 
     // load the plugin
-    KwavePlugin* plugin = loadPlugin(name);
-    if (!plugin) return;
+    KwavePlugin *plugin = loadPlugin(name);
+    if (!plugin) return -ENOMEM;
 
     if (params) {
 	// parameters were specified -> call directly
 	// without setup dialog
-	int result = plugin->start(*params);
+	result = plugin->start(*params);
 	
 	// maybe the start() function has called close() ?
 	if (m_loaded_plugins.findRef(plugin) == -1) {
@@ -308,14 +343,38 @@ void PluginManager::executePlugin(const QString &name, QStringList *params)
     // emit a command, let the toplevel window (and macro recorder) get
     // it and call us again later...
     if (command.length()) emit sigCommand(command);
+
+    return result;
 }
 
 //***************************************************************************
-void PluginManager::setupPlugin(const QString &name)
+void PluginManager::sync()
+{
+    bool one_is_running = true;
+    while (one_is_running) {
+	{
+//	    MutexGuard lock_list(m_lock_plugin_list);
+	    QListIterator<KwavePlugin> it(m_loaded_plugins);
+	    one_is_running = false;
+	    for (; it.current(); ++it) {
+		KwavePlugin *plugin = it.current();
+		if (plugin->isRunning()) {
+		    debug("waiting for plugin '%s'", plugin->name().data());
+		    one_is_running = true;
+		    break;
+		}
+	    }
+	}
+	if (one_is_running) qApp->processEvents();
+    }
+}
+
+//***************************************************************************
+int PluginManager::setupPlugin(const QString &name)
 {
     // load the plugin
     KwavePlugin* plugin = loadPlugin(name);
-    if (!plugin) return;
+    if (!plugin) return -ENOMEM;
 
     // now the plugin is present and loaded
     QStringList last_params = loadPluginDefaults(name, plugin->version());
@@ -327,12 +386,13 @@ void PluginManager::setupPlugin(const QString &name)
 	// the setup function has not been aborted.
 	savePluginDefaults(name, plugin->version(), *params);
 	delete params;
-    }
+    } else return -1;
 
     // now the plugin is no longer needed here, so delete it
     // if it has not already been detached and is not persistent
     if (!plugin->isPersistent()) pluginClosed(plugin, true);
 
+    return 0;
 }
 
 //***************************************************************************
@@ -418,10 +478,9 @@ unsigned int PluginManager::selectionEnd()
 }
 
 //***************************************************************************
-QBitmap *PluginManager::overview(unsigned int width, unsigned int height,
-                                 unsigned int offset, unsigned int length)
+void PluginManager::selectRange(unsigned int offset, unsigned int length)
 {
-    return m_top_widget.signalManager().overview(width,height,offset,length);
+    m_top_widget.signalManager().selectRange(offset, length);
 }
 
 //***************************************************************************
@@ -495,7 +554,8 @@ void PluginManager::pluginClosed(KwavePlugin *p, bool remove)
     ASSERT(!m_loaded_plugins.isEmpty());
 
     if (p) {
-//	void *h = p->getHandle();
+//	void *h = p->handle();
+	
 	// disconnect the signals to avoid recursion
 	disconnectPlugin(p);
 
@@ -579,7 +639,7 @@ void PluginManager::findPlugins()
 	    if (!*name || !*version || !*author) continue;
 	
 	    m_plugin_files.insert(*name, file);
-	    printf("%14s %5s written by %s", *name, *version, *author);
+	    printf("%16s %5s written by %s", *name, *version, *author);
 	
 	    dlclose (handle);
 	
