@@ -18,8 +18,6 @@
 #include "config.h"
 
 #include <errno.h>
-#include <endian.h>
-#include <byteswap.h>
 #include <limits.h>
 #include <math.h>
 
@@ -32,9 +30,14 @@
 
 #include <klocale.h>
 #include <kmessagebox.h>
+#include <kmimetype.h>
 #include <kprogress.h>
+#include <kurl.h>
 
-#include "libkwave/FileFormat.h"
+#include "mt/ThreadsafeX11Guard.h"
+
+#include "libkwave/Decoder.h"
+#include "libkwave/Encoder.h"
 #include "libkwave/InsertMode.h"
 #include "libkwave/MultiTrackReader.h"
 #include "libkwave/MultiTrackWriter.h"
@@ -46,11 +49,11 @@
 #include "libkwave/Track.h"
 
 #include "libgui/FileProgress.h"
-
-#include "mt/ThreadsafeX11Guard.h"
+#include "libgui/OverViewCache.h"
 
 #include "KwaveApp.h"
 #include "ClipBoard.h"
+#include "CodecManager.h"
 #include "SignalManager.h"
 #include "SignalWidget.h"
 #include "UndoAction.h"
@@ -83,7 +86,6 @@ SignalManager::SignalManager(QWidget *parent)
     m_modified_enabled(true),
     m_signal(),
     m_selection(0,0),
-    m_rate(0),
     m_last_length(0),
     m_playback_controller(),
     m_undo_enabled(false),
@@ -93,7 +95,8 @@ SignalManager::SignalManager(QWidget *parent)
     m_undo_transaction_level(0),
     m_undo_transaction_lock(),
     m_spx_undo_redo(this, SLOT(emitUndoRedoInfo())),
-    m_undo_limit(10*1024*1024) // 10 MB (for testing) ###
+    m_undo_limit(64*1024*1024) // 64 MB (for testing) ###
+    /** @todo the undo memory limit should be user-configurable. */
 {
     // connect to the track's signals
     Signal *sig = &m_signal;
@@ -116,11 +119,10 @@ SignalManager::SignalManager(QWidget *parent)
 }
 
 //***************************************************************************
-int SignalManager::loadFile(const QString &filename, int type)
+int SignalManager::loadFile(const KURL &url)
 {
     int res = 0;
-    ASSERT(filename.length());
-    m_name = filename;
+    m_name = url.filename();
 
     // enter and stay in not modified state
     enableModifiedChange(true);
@@ -130,18 +132,81 @@ int SignalManager::loadFile(const QString &filename, int type)
     // disable undo (discards all undo/redo data)
     disableUndo();
 
-    debug("SignalManager::loadFile(%s, %d)", filename.data(), type); // ###
-    switch (type) {
-	case WAV:
-	    res = loadWav();
+    QString mimetype = CodecManager::whatContains(url);
+    debug("SignalManager::loadFile(%s) - [%s]", url.prettyURL().data(),
+          mimetype.data());
+    Decoder *decoder = CodecManager::decoder(mimetype);
+    while (decoder) {
+	// be sure that the current signal is really closed
+	m_signal.close();
+	
+	// open the source file
+	QString filename = url.path();
+	QFile src(filename);
+	if (!decoder->open(m_parent_widget, src)) {
+	    warning("unable to open source: '%s'", url.prettyURL().data());
+	    res = -EIO;
 	    break;
-	case ASCII:
-	    res = loadAscii();
-	    break;
-	default:
-	    ASSERT("unknown file type");
-	    res = -EMEDIUMTYPE;
+	}
+	
+	// we must change to open state to see the file while
+	// it is loaded
+	m_closed = false;
+	m_empty = false;
+	
+	// set rate and resolution of the signal
+	m_signal.setRate((int)rint(decoder->info().rate()));
+	m_signal.setBits(decoder->info().bits());
+	
+	// create all tracks (empty)
+	unsigned int track;
+	const unsigned int tracks = decoder->info().tracks();
+	const unsigned int length = decoder->info().length();
+	ASSERT(tracks);
+	ASSERT(length);
+	for (track=0; track < tracks; ++track) {
+	    Track *t = m_signal.appendTrack(length);
+	    ASSERT(t);
+	    if (!t || t->length() != length) {
+		warning("out of memory");
+		res = -ENOMEM;
+		break;
+	    }
+	}
+	if (track < tracks) break;
+	
+	// create the multitrack writer as destination
+	MultiTrackWriter writers;
+	openMultiTrackWriter(writers, allTracks(), Overwrite, 0, length-1);
+	
+	//prepare and show the progress dialog
+	FileInfo &info = decoder->info();
+	FileProgress *dialog = new FileProgress(m_parent_widget,
+	    filename, info.tracks()*info.length(), info.length(),
+	    info.rate(), info.bits(), info.tracks());
+	ASSERT(dialog);
+	QObject::connect(&writers, SIGNAL(progress(unsigned int)),
+	                 dialog, SLOT(setValue(unsigned int)));
+	QObject::connect(dialog, SIGNAL(cancelled()),
+	                 &writers, SLOT(cancel()));
+	
+	// now decode
+	res = 0;
+	if (!decoder->decode(m_parent_widget, writers)) {
+	    warning("decoding failed.");
+	    res = -EIO;
+	}
+	decoder->close();
+	if (dialog) delete dialog;
+	
+	emitStatusInfo();
+	break;
     }
+    if (!decoder) {
+	warning("unknown file type");
+	res = -EMEDIUMTYPE;
+    }
+    if (decoder) delete decoder;
 
     // remember the last length
     m_last_length = length();
@@ -152,6 +217,96 @@ int SignalManager::loadFile(const QString &filename, int type)
     // modified can change from now on
     enableModifiedChange(true);
 
+    if (res) close();
+    return res;
+}
+
+//***************************************************************************
+int SignalManager::save(const KURL &url, unsigned int bits, bool selection)
+{
+    int res = 0;
+    unsigned int ofs = 0;
+    unsigned int len = length();
+    unsigned int tracks = this->tracks();
+
+    if (selection) {
+	// zero-length -> nothing to do
+	ofs = m_selection.offset();
+	len = m_selection.length();
+	tracks = selectedTracks().count();
+    }
+
+    if (!tracks || !len) {
+	KMessageBox::error(m_parent_widget,
+	    i18n("Signal is empty, nothing to save !"));
+	return 0;
+    }
+
+    QString mimetype_name;
+    mimetype_name = KMimeType::findByURL(url)->name();
+    debug("SignalManager::save(%s) - [%s] (%d bit, selection=%d)",
+	url.prettyURL().data(), mimetype_name.data(), bits, selection);
+
+    Encoder *encoder = CodecManager::encoder(mimetype_name);
+    if (encoder) {
+	// open the destination file
+	QString filename = url.path();
+	QFile dst(filename);
+	
+	MultiTrackReader src;
+	if (selection) {
+	    openMultiTrackReader(src, selectedTracks(), ofs, ofs+len-1);
+	} else {
+	    openMultiTrackReader(src, allTracks(), ofs, ofs+len-1);
+	}
+	
+	// create file information
+	FileInfo info;
+	info.setLength(len);
+	info.setRate(rate());
+	info.setBits(bits);
+	info.setTracks(tracks);
+	
+	//prepare and show the progress dialog
+	FileProgress *dialog = new FileProgress(m_parent_widget,
+	    filename, info.tracks()*info.length(), info.length(),
+	    info.rate(), info.bits(), info.tracks());
+	ASSERT(dialog);
+	QObject::connect(&src,   SIGNAL(progress(unsigned int)),
+	                 dialog, SLOT(setValue(unsigned int)));
+	QObject::connect(dialog, SIGNAL(cancelled()),
+	                 &src,   SLOT(cancel()));
+	
+	// invoke the encoder...
+	if (!encoder->encode(m_parent_widget, src, dst, info)) {
+	    KMessageBox::error(m_parent_widget,
+	        i18n("An error occurred while reading the file!"));
+	    res = -1;
+	}
+	
+	delete encoder;
+	if (dialog) {
+	    if (dialog->isCancelled()) {
+		// user really pressed cancel !
+		KMessageBox::error(m_parent_widget,
+		    i18n("The file has been truncated and "\
+		         "might be corrupted!"));
+	    }
+	    delete dialog;
+	}
+    } else {
+	KMessageBox::error(m_parent_widget,
+	    i18n("Sorry, the file type is not supported!"));
+	res = -EMEDIUMTYPE;
+    }
+
+    if (!res) {
+	// saved without error -> no longer modified
+	flushUndoBuffers();
+	enableModifiedChange(true);
+	setModified(false);
+    }
+    debug("SignalManager::save(): res=%d",res);
     return res;
 }
 
@@ -167,7 +322,7 @@ void SignalManager::newSignal(unsigned int samples, double rate,
     // disable undo (discards all undo/redo data)
     disableUndo();
 
-    m_rate = (int)rint(rate);
+    m_signal.setRate((int)rint(rate));
     m_signal.setBits(bits);
 
     // now the signal is considered not to be empty
@@ -182,7 +337,6 @@ void SignalManager::newSignal(unsigned int samples, double rate,
 
     // from now on, undo is enabled
     enableUndo();
-
 }
 
 //***************************************************************************
@@ -202,10 +356,10 @@ void SignalManager::close()
 
     m_empty = true;
     m_name = "";
+    while (tracks()) deleteTrack(tracks()-1);
     m_signal.close();
 
     m_closed = true;
-    m_rate = 0;
     m_selection.select(0,0);
     emitStatusInfo();
 }
@@ -232,60 +386,6 @@ const QArray<unsigned int> SignalManager::allTracks()
     return m_signal.allTracks();
 }
 
-//****************************************************************************
-QBitmap *SignalManager::overview(unsigned int width, unsigned int height,
-                                 unsigned int offset, unsigned int length)
-{
-    debug("SignalManager::overview(%u,%u,%u,%u)",width,height,offset,length);
-    QBitmap *overview = new QBitmap(width, height);
-    ASSERT(overview);
-    if (!overview) return 0;
-
-    unsigned int track;
-    unsigned int left;
-    unsigned int right;
-    unsigned int x;
-    unsigned int tracks = this->tracks();
-    double samples_per_pixel = (float)(length-1) / (float)(width-1);
-    int min;
-    int max;
-    double scale_y = (double)height / ((SAMPLE_MAX+1) << 1);
-
-    QPainter p;
-    MultiTrackReader src;
-    openMultiTrackReader(src, allTracks(), 0, length-1);
-
-    overview->fill(color0);
-
-    p.begin(overview);
-    p.setPen(color1);
-    left = offset;
-//    for (x=0; x < width; x++) {
-//	right = offset + (unsigned int)((x+1) * samples_per_pixel);
-//	// find minimum and maximum over all channels
-//	min = SAMPLE_MAX;
-//	max = SAMPLE_MIN;
-//	while (left++ <= right) {
-//	    for (track = 0; track < tracks; track++) {
-//		sample_t s;
-//		*(src.at(track)) >> s;
-//		if (s < min) min = s;
-//		if (s > max) max = s;
-//	    }
-//	}
-//	
-//	// transform min/max into pixel coordinates
-//	min = (height >> 1) - (int)(min * scale_y);
-//	max = (height >> 1) - (int)(max * scale_y);
-//	
-//	// draw the line between min and max
-//	p.drawLine(x, min, x, max);
-//    }
-    p.end ();
-
-    return overview;
-}
-
 //***************************************************************************
 SampleWriter *SignalManager::openSampleWriter(unsigned int track,
 	InsertMode mode, unsigned int left, unsigned int right,
@@ -364,7 +464,6 @@ void SignalManager::openMultiTrackWriter(MultiTrackWriter &writers,
     UndoTransactionGuard guard(*this, 0);
     unsigned int count = track_list.count();
     unsigned int track;
-    writers.setAutoDelete(true);
     writers.clear();
     writers.resize(count);
 
@@ -388,7 +487,7 @@ void SignalManager::openMultiTrackWriter(MultiTrackWriter &writers,
 //***************************************************************************
 bool SignalManager::executeCommand(const QString &command)
 {
-    debug("SignalManager::executeCommand(%s)", command.data());    // ###
+//    debug("SignalManager::executeCommand(%s)", command.data());    // ###
 
     unsigned int offset = m_selection.offset();
     unsigned int length = m_selection.length();
@@ -529,7 +628,6 @@ void SignalManager::appendTrack()
 void SignalManager::insertTrack(unsigned int index)
 {
     UndoTransactionGuard u(*this, i18n("insert track"));
-    debug("void SignalManager::insertTrack(%u)",index);
 
     UndoAction *undo = new UndoInsertTrack(m_signal, index);
     if (!registerUndoAction(undo)) {
@@ -548,11 +646,11 @@ void SignalManager::insertTrack(unsigned int index)
 
     if (index >= count) {
 	// do an "append"
-	debug("SignalManager::insertTrack(): appending");
+//	debug("SignalManager::insertTrack(): appending");
 	m_signal.appendTrack(len);
     } else {
 	// insert into the list
-	debug("m_signal.insertTrack(index, len);"); // ###
+	warning("m_signal.insertTrack(index, len) - NOT IMPLEMENTED !");
 	// ### TODO ### m_signal.insertTrack(index, len);
     }
 
@@ -564,7 +662,6 @@ void SignalManager::insertTrack(unsigned int index)
 void SignalManager::deleteTrack(unsigned int index)
 {
     UndoTransactionGuard u(*this, i18n("delete track"));
-    debug("void SignalManager::deleteTrack(%u)",index);
 
     UndoAction *undo = new UndoDeleteTrack(m_signal, index);
     if (!registerUndoAction(undo)) {
@@ -727,63 +824,66 @@ void SignalManager::selectTrack(unsigned int track, bool select)
 }
 
 //***************************************************************************
+/* ### -> should go to a "codec_ascii" plugin...
+
 int SignalManager::loadAscii()
 {
-//    float value;
-//    int cnt = 0;
-//    float max = 0;
-//    float amp;
-//    int *sample = 0;
-//
-//    FILE *sigin = fopen(m_name.data(), "r");
-//    if (!sigin) {
-//	KMessageBox::error(0, i18n("File does not exist !"), i18n("Info"), 2);
-//	return -ENOENT;
-//    }
-//
-//    // scan in the first line for the sample rate
-//    // ### to be done ###
-//    rate = 44100;        //will be asked in requester
-//
-//    // scan for number of channels
-//    m_channels = 1;
-//
-//    // loop over all samples in file to get maximum value
-//    while (!feof(sigin)) {
-//	if (fscanf (sigin, "%e\n", &value) == 1) {
-//	    if ( value > max) max = value;
-//	    if ( -value > max) max = -value;
-//	    cnt++;
-//	}
-//    }
-//    debug("SignalManager::loadAscii(): reading ascii file with %d samples",
-//	  cnt);    // ###
-//
-//    // get the maximum and the scale
-//    amp = (float)((1 << 23)-1) / max;
-//    Signal *new_signal = new Signal(cnt, rate);
-//    ASSERT(new_signal);
-//
-//    if (new_signal) {
-//	signal.append(new_signal);
-//	new_signal->setBits(24);
-//	sample = new_signal->getSample();
-//    }
-//
-//    if (sample) {
-//	fseek (sigin, 0, SEEK_SET);    //seek to beginning
-//	cnt = 0;
-//	while (!feof(sigin)) {
-//	    if (fscanf(sigin, "%e\n", &value) == 1) {
-//		sample[cnt++] = (int)(value * amp);
-//	    }
-//	}
-//    }
-//
-//    fclose (sigin);
+    float value;
+    int cnt = 0;
+    float max = 0;
+    float amp;
+    int *sample = 0;
+
+    FILE *sigin = fopen(m_name.data(), "r");
+    if (!sigin) {
+	KMessageBox::error(0, i18n("File does not exist !"), i18n("Info"), 2);
+	return -ENOENT;
+    }
+
+    // scan in the first line for the sample rate
+    // ### to be done ###
+    rate = 44100;        //will be asked in requester
+
+    // scan for number of channels
+    m_channels = 1;
+
+    // loop over all samples in file to get maximum value
+    while (!feof(sigin)) {
+	if (fscanf (sigin, "%e\n", &value) == 1) {
+	    if ( value > max) max = value;
+	    if ( -value > max) max = -value;
+	    cnt++;
+	}
+    }
+    debug("SignalManager::loadAscii(): reading ascii file with %d samples",
+	  cnt);    // ###
+
+    // get the maximum and the scale
+    amp = (float)((1 << 23)-1) / max;
+    Signal *new_signal = new Signal(cnt, rate);
+    ASSERT(new_signal);
+
+    if (new_signal) {
+	signal.append(new_signal);
+	new_signal->setBits(24);
+	sample = new_signal->getSample();
+    }
+
+    if (sample) {
+	fseek (sigin, 0, SEEK_SET);    //seek to beginning
+	cnt = 0;
+	while (!feof(sigin)) {
+	    if (fscanf(sigin, "%e\n", &value) == 1) {
+		sample[cnt++] = (int)(value * amp);
+	    }
+	}
+    }
+
+    fclose (sigin);
 
     return 0;
 }
+### */
 
 //***************************************************************************
 void SignalManager::emitStatusInfo()
@@ -791,591 +891,73 @@ void SignalManager::emitStatusInfo()
     emit sigStatusInfo(length(), tracks(), rate(), bits());
 }
 
-//***************************************************************************
-int SignalManager::loadWav()
+/* ### -> should go to a "codec_ascii" plugin...
+
+int SignalManager::exportAscii(const char *name)
 {
-    wav_fmt_header_t fmt_header;
-    int result = 0;
-    __uint32_t num;
-    __uint32_t length;
+    ASSERT(name);
+    if (!name) return ;
 
-    ASSERT(m_closed);
-    ASSERT(m_empty);
-
-    QFile sigfile(m_name);
-    if (!sigfile.open(IO_ReadOnly)) {
-	KMessageBox::error(m_parent_widget,
-		i18n("File does not exist !"));
-	return -ENOENT;
-    }
-
-    // --- check if the file starts with "RIFF" ---
-    num = sigfile.size();
-    length = findChunk(sigfile, "RIFF", 0);
-    if ((length == 0) || (sigfile.at() != 8)) {
-	KMessageBox::error(m_parent_widget,
-	    i18n("File is no RIFF File !"));
-	// maybe recoverable...
-    } else if (length+8 != num) {
-//	KMessageBox::error(m_parent_widget,
-//	    i18n("File has incorrect length! (maybe truncated?)"));
-	// will be warned anyway later...
-	// maybe recoverable...
-    } else {
-	// check if the chunk data contains "WAVE"
-	char file_type[16];
-	num = sigfile.readBlock((char*)(&file_type), 4);
-	if ((num != 4) || strncmp("WAVE", file_type, 4)) {
-	    KMessageBox::error(m_parent_widget,
-		i18n("File is no WAVE File !"));
-	    // maybe recoverable...
-	}
-    }
-
-    // ------- read the "fmt " chunk -------
-    ASSERT(sizeof(fmt_header) == 16);
-    num = findChunk(sigfile, "fmt ");
-    if (num != sizeof(fmt_header)) {
-	debug("SignalManager::loadWav(): length of fmt chunk = %d", num);
-	KMessageBox::error(m_parent_widget,
-	    i18n("File does not contain format information!"));
-	return -EMEDIUMTYPE;
-    }
-    num = sigfile.readBlock((char*)(&fmt_header), sizeof(fmt_header));
-#ifdef IS_BIG_ENDIAN
-    fmt_header.length = bswap_32(fmt_header.length);
-    fmt_header.mode = bswap_16(fmt_header.mode);
-    fmt_header.channels = bswap_16(fmt_header.channels);
-    fmt_header.rate = bswap_32(fmt_header.rate);
-    fmt_header.AvgBytesPerSec = bswap_32(fmt_header.AvgBytesPerSec);
-    fmt_header.BlockAlign = bswap_32(fmt_header.BlockAlign);
-    fmt_header.bitspersample = bswap_16(fmt_header.bitspersample);
-#endif
-    if (fmt_header.mode != 1) {
-	KMessageBox::error(m_parent_widget,
-	    i18n("File must be uncompressed (Mode 1) !"),
-	    i18n("Sorry"), 2);
-	return -EMEDIUMTYPE;
-    }
-
-    m_rate = fmt_header.rate;
-    m_signal.setBits(fmt_header.bitspersample);
-
-    // ------- search for the data chunk -------
-    length = findChunk(sigfile, "data");
-    if (!length) {
-	warning("length = 0, but file size = %u, current pos = %u",
-	    sigfile.size(), sigfile.at());
-	
-	if (sigfile.size() - sigfile.at() > 0) {
-	    if (KMessageBox::warningContinueCancel(m_parent_widget,
-		i18n("File is damaged: the file header reports zero length\n"\
-		     "but the file seems to contain some data. \n\n"\
-		     "Kwave can try to recover the file, but the\n"\
-		     "result might contain trash at it's start and/or\n"\
-		     "it's end. It is strongly advisable to edit the\n"\
-		     "damaged parts manually and save the file again\n"\
-		     "to fix the problem.\n\n"\
-		     "Try to recover?"),
-		     i18n("Damaged File"),
-		     i18n("&Recover")) == KMessageBox::Continue)
-	    {
-		length = sigfile.size() - sigfile.at();
-	    } else return -EMEDIUMTYPE;
-	} else {
-	    KMessageBox::error(m_parent_widget,
-	        i18n("File does not contain data!"));
-	    return -EMEDIUMTYPE;
-	}
-    }
-
-    length = (length/(fmt_header.bitspersample/8))/fmt_header.channels;
-    switch (fmt_header.bitspersample) {
-	case 8:
-	case 16:
-	case 24:
-	    ASSERT(m_empty);
-	    // currently the signal should be closed and empty
-	    // now make it opened but empty
-	    m_closed = false;
-	    emitStatusInfo();
-	    result = loadWavChunk(sigfile, length,
-				  fmt_header.channels,
-				  fmt_header.bitspersample);
-	    break;
-	default:
-	    KMessageBox::error(m_parent_widget,
-		i18n("Sorry only 8/16/24 Bits per Sample"\
-		" are supported !"), i18n("Sorry"), 2);
-	    result = -EMEDIUMTYPE;
-    }
-
-    if (result == 0) {
-	debug("SignalManager::loadWav(): successfully opened");
-    }
-
-    return result;
-}
-
-//**********************************************************
-// the following routines are for loading and saving in dataformats
-// specified by names little/big endian problems are dealt with at compile time
-// The corresponding header should have already been written to the file before
-// invocation of this methods
-//***************************************************************************
-int SignalManager::exportAscii(const char */*name*/)
-{
-//    ASSERT(name);
-//    if (!name) return ;
-//
-//    unsigned int length = getLength();
-//    ASSERT(length);
-//    if (!length) return;
-//
-//    ASSERT(m_channels);
-//    if (!m_channels) return;
-//
-//    FILE *sigout = fopen(name, "w");
-//    ASSERT(sigout);
-//    if (!sigout) return;
-//
-//    //prepare and show the progress dialog
-//    char progress_title[256];
-//    char str_channels[128];
-//    if (m_channels == 1)
-//	strncpy(str_channels, i18n("Mono"), sizeof(str_channels));
-//    else if (m_channels == 2)
-//	strncpy(str_channels, i18n("Stereo"), sizeof(str_channels));
-//    else
-//	snprintf(str_channels, sizeof(str_channels), "%d-channel", m_channels);
-//    snprintf(progress_title, sizeof(progress_title),
-//	i18n("Exporting %s ASCII file :"),
-//	str_channels);
-//
-//    QString title = i18n(progress_title);
-//    ProgressDialog *dialog = new ProgressDialog(100, title);
-//    delete[] title;
-//    if (dialog) dialog->show();
-//
-//    // loop for writing data
-//    int *sample;
-//    int percent_count = 0;
-//    const double scale_y = (1 << 23)-1;
-//    for (unsigned int pos = 0; pos < length ; pos++) {
-//	// loop over all channels
-//	for (unsigned int channel=0; channel < m_channels; channel++) {
-//	    sample = signal.at(channel)->getSample();
-//	    if (!sample) continue;
-//	
-//	    if (channel != 0) fprintf(sigout, ",");
-//	    fprintf(sigout, "%0.8e", (double)sample[pos]/scale_y);
-//	}
-//	fprintf(sigout,"\n");
-//
-//	// update the progress bar
-//	percent_count--;
-//	if (dialog && (percent_count <= 0)) {
-//	    percent_count = length / 200;
-//	    float percent = (float)pos;
-//	    percent /= (float)length;
-//	    percent *= 100.0;
-//	    dialog->setProgress (percent);
-//	}
-//    }
-//
-//    if (dialog) delete dialog;
-//    fclose(sigout);
-    return -1;
-}
-
-//***************************************************************************
-int SignalManager::writeWavChunk(QFile &sigout, unsigned int offset,
-                                 unsigned int length, unsigned int bits)
-{
-    unsigned int bufsize = 16 * 1024 * sizeof(int);
-    unsigned char *savebuffer = 0;
-    int bytes = bits / 8;
-    unsigned int tracks = this->tracks();
-    int bytes_per_sample = bytes * tracks;
-    bufsize -= bufsize % bytes_per_sample;
-
-    // try to allocate memory for the save buffer
-    // if failed, try again with the half buffer size as long
-    // as <1kB is not reached (then we are really out of memory)
-    while (savebuffer == 0) {
-	if (bufsize < 1024) {
-	    warning("SignalManager::writeWavChunk:no memory for buffer");
-	    return -ENOMEM;
-	}
-	savebuffer = new unsigned char[bufsize];
-	if (!savebuffer) {
-	    bufsize >>= 1;
-	    bufsize -= bufsize % bytes_per_sample;
-	}
-    }
-
-    // prepare and show the progress dialog
-    unsigned int file_rest = tracks * length * bytes_per_sample;
-    FileProgress *dialog = new FileProgress(m_parent_widget,
-	m_name, file_rest, length, m_rate, bits, tracks);
-    ASSERT(dialog);
-
-    // prepare the store loop
-    int percent_count = length / 200;
-    unsigned int shift = 24-bits;
-
-    QList<SampleReader> samples;
-    samples.setAutoDelete(true);
-    for (unsigned int track = 0; track < tracks; track++) {
-	SampleReader *s = m_signal.openSampleReader(track,
-	    offset, offset+length-1);
-	ASSERT(s);
-	if (!s) {
-	    KMessageBox::sorry(m_parent_widget, i18n("Out of Memory!"));
-	    return -ENOMEM;
-	}
-	samples.append(s);
-    }
-
-    // loop for writing data
-    for (unsigned int pos = offset; pos < offset+length; ) {
-	unsigned char *buf = savebuffer;
-	unsigned int nsamples = 0;
-	
-	// break the loop if the user has pressed "cancel"
-	if (dialog && dialog->isCancelled()) break;
-	
-	while (pos < offset+length &&
-	      (nsamples < (bufsize/bytes_per_sample)))
-	{
-	    for (unsigned int track = 0; track < tracks; track++) {
-		SampleReader *stream = samples.at(track);
-		sample_t sample;
-		(*stream) >> sample;
-		
-		// the following cast is only necessary if
-		// sample_t is not equal to a 32bit int
-		__uint32_t act = static_cast<__uint32_t>(sample);
-		
-		act >>= shift;
-		if (bytes == 1) {
-		    // 8 bits -> unsigned
-		    *(buf++) = (char)((act - 128) & 0xFF);
-		} else {
-		    // >= 16 bits -> signed
-		    for (register int byte = bytes; byte; byte--) {
-			*(buf++) = (char)(act & 0xFF);
-			act >>= 8;
-		    }
-		}
-	    }
-	    nsamples++;
-	    pos++;
-	}
-	
-	int written_bytes = sigout.writeBlock(
-	    reinterpret_cast<char *>(savebuffer),
-	    bytes_per_sample * nsamples);
-	
-	percent_count -= written_bytes;
-	if (dialog && (percent_count <= 0)) {
-	    percent_count = length / 200;
-	    dialog->setValue(pos*tracks*bytes_per_sample);
-	}
-    }
-
-    int res = 0;
-    if (dialog && dialog->isCancelled()) res = -1;
-
-    if (dialog) delete dialog;
-    if (savebuffer) delete[] savebuffer;
-
-    // close all SampleReaders
-    samples.clear();
-
-    return res;
-}
-
-//***************************************************************************
-int SignalManager::save(const QString &filename, unsigned int bits,
-                         bool selection)
-{
-    debug("SignalManager::save(): %u Bit to %s ,%d",
-    	bits, filename.data(), selection);
-
-    int res = 0;
-    __uint32_t ofs = 0;
-    __uint32_t len = length();
-    unsigned int tracks = this->tracks();
-    wav_header_t header;
-
-    ASSERT(filename);
-    if (!filename) return -EINVAL;
-
-    if (selection) {
-	// zero-length -> nothing to do
-	if (m_selection.length() == 0) return 0;
-	ofs = m_selection.offset();
-	len = m_selection.length();
-    }
-
-    if (!tracks || !len) {
-	KMessageBox::error(m_parent_widget,
-	    i18n("Signal is empty, nothing to save !"));
-	return 0;
-    }
-
-    QFile sigout(filename);
-    if (!(res = sigout.open(IO_WriteOnly | IO_Truncate))) {
-	KMessageBox::error(m_parent_widget,
-	    i18n("Opening the file for writing failed!"));
-	return -res;
-    };
-
-    __uint32_t datalen = (bits >> 3) * len * tracks;
-    strncpy((char*)&(header.riffid), "RIFF", 4);
-    strncpy((char*)&(header.wavid), "WAVE", 4);
-    strncpy((char*)&(header.fmtid), "fmt ", 4);
-    header.filelength = datalen + sizeof(wav_header_t);
-    header.fmtlength = 16;
-    header.mode = 1;
-    header.channels = tracks;
-    header.rate = m_rate;
-    header.AvgBytesPerSec = m_rate * (bits >> 3) * tracks;
-    header.BlockAlign = (bits >> 3) * tracks;
-    header.bitspersample = bits;
-
-#if defined(IS_BIG_ENDIAN)
-    header.filelength = bswap_32(header.filelength);
-    header.fmtlength = bswap_32(header.fmtlength);
-    header.mode = bswap_16(header.mode);
-    header.channels = bswap_16(header.channels);
-    header.rate = bswap_32(header.rate);
-    header.AvgBytesPerSec = bswap_32(header.AvgBytesPerSec);
-    header.BlockAlign = bswap_16(header.BlockAlign);
-    header.bitspersample = bswap_16(header.bitspersample);
-    datalen = bswap_32(datalen);
-#endif
-
-    sigout.at(0);
-    sigout.writeBlock((char *) &header, sizeof(wav_header_t));
-    sigout.writeBlock("data", 4);
-    sigout.writeBlock((char *)&datalen, 4);
-
-    switch (bits) {
-	case 8:
-	case 16:
-	case 24:
-	    res = writeWavChunk(sigout, ofs, len, bits);
-	    break;
-	default:
-	    KMessageBox::sorry(m_parent_widget,
-		i18n("Sorry only 8/16/24 Bits per Sample are supported !"));
-	    sigout.close();
-	    res = -1;
-	break;
-    }
-
-    sigout.close();
-    if (!res) {
-	// saved without error -> no longer modified
-	flushUndoBuffers();
-	enableModifiedChange(true);
-	setModified(false);
-    }
-    debug("SignalManager::save(): res=%d",res);
-    return res;
-}
-
-//***************************************************************************
-__uint32_t SignalManager::findChunk(QFile &sigfile, const char *chunk,
-	__uint32_t offset)
-{
-    char current_name[16];
-    __uint32_t length = 0;
-    int len;
-
-    ASSERT(sizeof(length) == 4);
-    ASSERT(sizeof(int) == 4);
-
-    sigfile.at(offset);
-    while (!sigfile.atEnd()) {
-	// get name of the chunk
-	len = sigfile.readBlock((char*)(&current_name), 4);
-	if (len < 4) {
-	    debug("findChunk('%s'): not found, reached EOF while reading name",
-	    	chunk);
-	    return 0; // reached EOF
-	}
-
-	// get length of the chunk
-	len = sigfile.readBlock((char*)(&length), sizeof(length));
-	if (len < 4) {
-	    debug("findChunk('%s'): not found, reached EOF :-(", chunk);
-	    return 0; // reached EOF
-	}
-#ifdef IS_BIG_ENDIAN
-	length = bswap_32(length);
-#endif
-
-	// chunk found !
-	if (strncmp(chunk, current_name, 4) == 0) return length;
-
-	// not found -> skip
-	sigfile.at(sigfile.at()+length);
-    };
-
-    debug("findChunk('%s'): not found :-(", chunk);
-    return 0;
-}
-
-//***************************************************************************
-int SignalManager::loadWavChunk(QFile &sigfile, unsigned int length,
-                                unsigned int channels, int bits)
-{
-    unsigned int bufsize = 64 * 1024 * sizeof(sample_t);
-    unsigned char *loadbuffer = 0;
-    int bytes = bits >> 3;
-    unsigned int sign = 1 << (24-1);
-    unsigned int negative = ~(sign - 1);
-    unsigned int shift = 24-bits;
-    unsigned int bytes_per_sample = bytes * channels;
-    unsigned int max_samples = bufsize / bytes_per_sample;
-    long int start_offset = sigfile.at();
-
-    debug("SignalManager::loadWavChunk(): offset     = %d", sigfile.at());
-    debug("SignalManager::loadWavChunk(): length     = %d samples", length);
-    debug("SignalManager::loadWavChunk(): tracks     = %d", channels);
-    debug("SignalManager::loadWavChunk(): resoultion = %d bits/sample", bits);
-
-    ASSERT(bytes);
-    ASSERT(channels);
+    unsigned int length = getLength();
     ASSERT(length);
-    if (!bytes || !channels || !length) return -EINVAL;
+    if (!length) return;
 
-    // try to allocate memory for the load buffer
-    // if failed, try again with the half buffer size as long
-    // as <1kB is not reached (then we are really out of memory)
-    while (loadbuffer == 0) {
-	if (bufsize < 1024) {
-	    debug("SignalManager::loadWavChunk:not enough memory for buffer");
-	    return -ENOMEM;
-	}
-	loadbuffer = new unsigned char[bufsize];
-	if (!loadbuffer) bufsize >>= 1;
-    }
+    ASSERT(m_channels);
+    if (!m_channels) return;
 
-    // check if the file is large enough for "length" samples
-    size_t file_rest = sigfile.size() - sigfile.at();
-    if (length > file_rest/bytes_per_sample) {
-	debug("SignalManager::loadWavChunk: "\
-	      "length=%d, rest of file=%d",length,file_rest);
-	KMessageBox::error(m_parent_widget,
-	    i18n("Error in input: file is smaller than stated "\
-	    "in the header. \n"\
-	    "File will be truncated."));
-	length = file_rest/bytes_per_sample;
-    }
-
-    QList<SampleWriter> samples;
-    samples.setAutoDelete(true);
-
-    for (unsigned int track = 0; track < channels; track++) {
-	SampleWriter *s = 0;
-	Track *new_track = m_signal.appendTrack(length);
-	ASSERT(new_track);
-	if (new_track && (new_track->length() >= length)) {
-	    s = openSampleWriter(track, Overwrite);
-	    ASSERT(s);
-	}
-	
-	if (!s) {
-	    KMessageBox::sorry(m_parent_widget, i18n("Out of Memory!"));
-	    return -ENOMEM;
-	}
-	samples.append(s);
-    }
-
-    // now the signal is considered not to be empty
-    m_empty = false;
+    FILE *sigout = fopen(name, "w");
+    ASSERT(sigout);
+    if (!sigout) return;
 
     //prepare and show the progress dialog
-    FileProgress *dialog = new FileProgress(m_parent_widget,
-	m_name, file_rest, length, m_rate, bits, channels);
-    ASSERT(dialog);
+    char progress_title[256];
+    char str_channels[128];
+    if (m_channels == 1)
+	strncpy(str_channels, i18n("Mono"), sizeof(str_channels));
+    else if (m_channels == 2)
+	strncpy(str_channels, i18n("Stereo"), sizeof(str_channels));
+    else
+	snprintf(str_channels, sizeof(str_channels), "%d-channel", m_channels);
+    snprintf(progress_title, sizeof(progress_title),
+	i18n("Exporting %s ASCII file :"),
+	str_channels);
 
-    // prepare the loader loop
-    int percent_count = length / 100;
+    QString title = i18n(progress_title);
+    ProgressDialog *dialog = new ProgressDialog(100, title);
+    delete[] title;
+    if (dialog) dialog->show();
 
-    // debug("sign=%08X, negative=%08X, shift=%d",sign,negative,shift);
-
-    for (unsigned int pos = 0; pos < length; ) {
-	// break the loop if the user has pressed "cancel"
-	if (dialog && dialog->isCancelled()) break;
+    // loop for writing data
+    int *sample;
+    int percent_count = 0;
+    const double scale_y = (1 << 23)-1;
+    for (unsigned int pos = 0; pos < length ; pos++) {
+	// loop over all channels
+	for (unsigned int channel=0; channel < m_channels; channel++) {
+	    sample = signal.at(channel)->getSample();
+	    if (!sample) continue;
 	
-	// limit reading to end of wav chunk length
-	if ((pos + max_samples) > length) max_samples=length-pos;
-	
-	// read the samples into a temporary buffer
-	int read_samples = sigfile.readBlock(
-	    reinterpret_cast<char *>(loadbuffer),
-	    bytes_per_sample*max_samples
-	) / bytes_per_sample;
-	percent_count -= read_samples;
-	
-	// debug("read %d samples", read_samples);
-	if (read_samples <= 0) {
-	    warning("SignalManager::loadWavChunk:EOF reached?"\
-		    " (at sample %ld, expected length=%d",
-		    sigfile.at() / bytes_per_sample - start_offset, length);
-	    break;
+	    if (channel != 0) fprintf(sigout, ",");
+	    fprintf(sigout, "%0.8e", (double)sample[pos]/scale_y);
 	}
-	
-	unsigned char *buffer = loadbuffer;
-	__uint32_t s = 0; // raw 32bit value
-	while (read_samples--) {
-	    for (register unsigned int channel = 0;
-		 channel < channels;
-		channel++)
-	    {
-		SampleWriter *stream = samples.at(channel);
-		
-		if (bytes == 1) {
-		    // 8-bit files are always unsigned !
-		    s = (*(buffer++) - 128) << shift;
-		} else {
-		    // >= 16 bits is signed
-		    s = 0;
-		    for (register int byte = 0; byte < bytes; byte++) {
-			s |= *(buffer++) << ((byte << 3) + shift);
-		    }
-		    // sign correcture for negative values
-		    if ((unsigned int)s & sign)
-			s |= negative;
-		}
-		
-		// the following cast is only necessary if
-		// sample_t is not equal to a 32bit int
-		sample_t sample = static_cast<sample_t>(s);
-		
-		*stream << sample;
-	    }
-	    pos++;
-	}
-	
+	fprintf(sigout,"\n");
+
+	// update the progress bar
+	percent_count--;
 	if (dialog && (percent_count <= 0)) {
-	    percent_count = length / 100;
-	    dialog->setValue(pos * bytes_per_sample);
+	    percent_count = length / 200;
+	    float percent = (float)pos;
+	    percent /= (float)length;
+	    percent *= 100.0;
+	    dialog->setProgress (percent);
 	}
     }
 
-    // close all sample input streams
-    samples.clear();
-
     if (dialog) delete dialog;
-    if (loadbuffer) delete[] loadbuffer;
-    return 0;
+    fclose(sigout);
+    return -1;
 }
+### */
 
 //***************************************************************************
 PlaybackController &SignalManager::playbackController()
@@ -1467,7 +1049,7 @@ void SignalManager::flushUndoBuffers()
     m_undo_transaction_level = 0;
 
     // if the signal was modified, it will stay in this state, it is
-    // not possible it to "non-modified" state through undo
+    // not possible to change to "non-modified" state through undo
     if ((!m_undo_buffer.isEmpty()) && (m_modified)) {
 	enableModifiedChange(false);
     }
@@ -1509,78 +1091,33 @@ bool SignalManager::registerUndoAction(UndoAction *action)
     // if undo is not enabled, return a faked "ok"
     if (!m_undo_enabled) return true;
 
-    ASSERT(m_undo_transaction);
-    if (!m_undo_transaction) return false;
-
     unsigned int needed_size  = action->undoSize();
     unsigned int needed_mb = needed_size  >> 20;
     unsigned int limit_mb  = m_undo_limit >> 20;
 
-    // Print a warning if the needed memory exceeds the limit for undo.
-    // The user should have the chance to decide between:
-    // a) abort the current operation
-    // b) permit the operation, but continue without undo
-    // c) ignore the memory limit and store undo data nevertheless
     if (needed_mb > limit_mb) {
-	ThreadsafeX11Guard x11_guard;
-	int choice = KMessageBox::warningYesNoCancel(m_parent_widget,
-	    /* HTML version */
-	    QString("<HTML>"+
-	    i18n("The operation '%1' requires more than %2 MB for "\
-	    "storing undo data.\n"\
-	    "This would exceed the limit of %3 MB.")+
-	    "<BR><BR>"+
-	    i18n("You can now either press:")+
-	    "<TABLE NOBORDER><TR>"\
-	    "<TD><B>"+
-	    i18n("Allow")+"</B></TD><TD>"+
-	    i18n("to continue without the possibility to undo, or")+
-	    "</TD></TR><TR>"\
-	    "<TD><B>"+i18n("Ignore")+"</B></TD><TD>"+
-	    i18n("to continue, discard all previous undo data "\
-	                    "and ignore the memory limit")+
-	    "</TD></TR><TR>"\
-	    "<td><B>"+
-	    i18n("Cancel")+
-	    "</B></TD><TD>"+
-	    i18n("to abort the current operation.")+
-	    "</td></TR></TABLE></HTML>").arg(
-	    m_undo_transaction->description()).arg(needed_mb).arg(limit_mb),
+	// Allow: discard buffers and omit undo
+	m_undo_buffer.clear();
+	m_redo_buffer.clear();
 	
-	    /* plain text version
-	    i18n("The command '%1' requires %2 MB for storing undo data.\n"\
-	    "This would exceed the limit of %3 MB. \n\n"\
-	    "You can now either press:\n"\
-	    "* <Allow>  to continue without the possibility to undo, or\n"\
-	    "* <Ignore> to continue and ignore the memory limit, or\n"\
-	    "* <Cancel> to abort the current operation.").arg(
-	    m_undo_transaction->description()).arg(needed_mb).arg(limit_mb),
-	    */
+	// close the current transaction
+	if (m_undo_transaction) delete m_undo_transaction;
+	m_undo_transaction = 0;
 	
-	    QString::null,
-	    i18n("&Allow"),
-	    i18n("&Ignore"),
-	    true
-	);
-	
-	switch (choice) {
-	    case KMessageBox::Yes:
-		// Allow: discard buffers and omit undo
-		m_undo_buffer.clear();
-		m_redo_buffer.clear();
-		return true;
-	    case KMessageBox::No:
-		// Ignore: discard buffers and ignore limit
-		m_undo_buffer.clear();
-		m_redo_buffer.clear();
-		break;
-	    default:
-		// Cancel: abort the action
-		return false;
+	// if the signal was modified, it will stay in this state, it is
+	// not possible to change to "non-modified" state through undo
+	if ((!m_undo_buffer.isEmpty()) && (m_modified)) {
+	    enableModifiedChange(false);
 	}
+	
+	m_spx_undo_redo.AsyncHandler();
+	return true;
     }
 
-    // now make room...
+    // undo has been aborted before ?
+    if (!m_undo_transaction) return true;
+
+    // make room...
     freeUndoMemory(needed_size);
 
     // now we have enough place to append the undo action
@@ -1614,7 +1151,7 @@ bool SignalManager::saveUndoDelete(QArray<unsigned int> &track_list,
     };
 
     // do not delete the actions from the list, so it's important
-    // disable the auto-delete feature now!
+    // to disable the auto-delete feature now!
     undo_list.setAutoDelete(false);
     return true;
 }
@@ -1649,7 +1186,7 @@ void SignalManager::freeUndoMemory(unsigned int needed)
 	m_undo_buffer.removeFirst();
 
 	// if the signal was modified, it will stay in this state, it is
-	// not possible it to "non-modified" state through undo
+	// not possible to change to "non-modified" state through undo
 	if ((!m_undo_buffer.isEmpty()) && (m_modified)) {
 	    enableModifiedChange(false);
 	}
