@@ -29,7 +29,7 @@
 #include <klocale.h>
 #include <kfiledialog.h>
 
-#include "libkwave/FileFormat.h"
+#include "libkwave/FileInfo.h"
 #include "libkwave/KwaveDrag.h"
 #include "libkwave/Label.h"
 #include "libkwave/LabelList.h"
@@ -69,14 +69,6 @@
  */
 #define MINIMUM_SAMPLES_PER_SCREEN 5
 
-#ifndef min
-#define min(x,y) (( (x) < (y) ) ? (x) : (y) )
-#endif
-
-#ifndef max
-#define max(x,y) (( (x) > (y) ) ? (x) : (y) )
-#endif
-
 //***************************************************************************
 //***************************************************************************
 class KwaveFileDrag: public QUriDrag
@@ -107,7 +99,6 @@ SignalWidget::SignalWidget(QWidget *parent)
     :QWidget(parent),
     m_inhibit_repaint(0),
     m_signal_manager(this),
-    m_refresh_timer(),
     m_track_pixmaps(),
     m_mouse_mode(MouseNormal),
     m_mouse_down_x(0)
@@ -212,8 +203,6 @@ SignalWidget::~SignalWidget()
 {
     close();
 
-    m_refresh_timer.stop();
-
     if (m_pixmap == 0) delete m_pixmap;
     m_pixmap = 0;
 
@@ -229,21 +218,9 @@ SignalWidget::~SignalWidget()
 }
 
 //***************************************************************************
-int SignalWidget::saveFile(const QString &filename, unsigned int bits,
-                           int type, bool selection)
+int SignalWidget::saveFile(const KURL &url, unsigned int bits, bool selection)
 {
-    if (type == ASCII) {
-	return m_signal_manager.exportAscii(filename);
-    } else {
-	return m_signal_manager.save(filename, bits, selection);
-    }
-}
-
-//***************************************************************************
-QBitmap *SignalWidget::overview(unsigned int width, unsigned int height)
-{
-    return m_signal_manager.overview(width, height, 0,
-                                     m_signal_manager.length());
+    return m_signal_manager.save(url, bits, selection);
 }
 
 //***************************************************************************
@@ -431,14 +408,13 @@ SignalManager &SignalWidget::signalManager()
 }
 
 //***************************************************************************
-int SignalWidget::loadFile(const QString &filename, int type)
+int SignalWidget::loadFile(const KURL &url)
 {
     // close the previous signal
     close();
 
     // load a new signal
-    int res = m_signal_manager.loadFile(filename, type);
-    ASSERT(m_signal_manager.length());
+    int res = m_signal_manager.loadFile(url);
     if (m_signal_manager.isClosed() || (res)) {
 	warning("SignalWidget::loadFile() failed:"\
 		" zero-length or out of memory?");
@@ -1139,9 +1115,6 @@ void SignalWidget::paintEvent(QPaintEvent *)
 //    debug("SignalWidget::paintEvent() -- done, t=%0.3fms --",
 //	t_elapsed); // ###
 //#endif
-
-//    // restart the timer for refreshing the playpointer
-//    if (m_playback_controller.running()) playback_startTimer();
 }
 
 //***************************************************************************
@@ -1172,6 +1145,12 @@ int SignalWidget::samples2pixels(int samples)
     return (int)rint((double)samples / m_zoom);
 }
 
+//***************************************************************************
+int SignalWidget::displaySamples()
+{
+    return pixels2samples(width()-1);
+}
+
 //***************************************************************************
 void selectMarkers(const char */*command*/)
 {
@@ -1729,7 +1708,7 @@ void SignalWidget::slotTrackDeleted(unsigned int index)
 {
     // delete the track from the list
     m_track_pixmaps.setAutoDelete(true);
-    m_track_pixmaps.remove(index);
+    if (index < m_track_pixmaps.count()) m_track_pixmaps.remove(index);
 
     // emit the signal sigTrackInserted now, so that the signal widget
     // gets resized if needed, but the new pixmap is still empty
@@ -1780,7 +1759,15 @@ void SignalWidget::startDragging()
 
     m_signal_manager.openMultiTrackReader(src,
 	m_signal_manager.selectedTracks(), first, last);
-    if (!d->encode(rate, bits, src)) {
+
+    // create the file info
+    FileInfo info;
+    info.setLength(last - first + 1);
+    info.setRate(rate);
+    info.setBits(bits);
+    info.setTracks(src.count());
+
+    if (!d->encode(this, src, info)) {
 	delete d;
 	return;
     }
@@ -1793,7 +1780,14 @@ void SignalWidget::startDragging()
 	const unsigned int f = m_signal_manager.selection().first();
 	const unsigned int l = m_signal_manager.selection().last();
 	const unsigned int len = l-f+1;
-	m_signal_manager.deleteRange(first, len,
+	
+	// special case: when dropping into the same widget, before
+	// the previous selection, the previous range has already
+	// been moved to the right !
+	unsigned int src = first;
+	if ((d->target() == this) && (f < src)) src += len;
+	
+	m_signal_manager.deleteRange(src, len,
 		m_signal_manager.selectedTracks());
 		
 	// restore the new selection
@@ -1823,10 +1817,8 @@ void SignalWidget::dropEvent(QDropEvent* event)
 	MultiTrackReader src;
 	MultiTrackWriter dst;
 	Signal sig;
-	unsigned int rate = 0;
-	unsigned int bits = 0;
 	
-	if (KwaveDrag::decode(event, sig, rate, bits)) {
+	if (KwaveDrag::decode(this, event, sig)) {
 	    InhibitRepaintGuard inhibit(*this);
 	    unsigned int pos = m_offset + pixels2samples(event->pos().x());
 	    unsigned int len = sig.length();
