@@ -18,6 +18,7 @@
 #ifndef _SIGNAL_WIDGET_H_
 #define _SIGNAL_WIDGET_H_
 
+#include "config.h"
 #include <qdragobject.h>
 #include <qlist.h>
 #include <qtimer.h>
@@ -27,6 +28,7 @@
 #include "PlaybackController.h"
 #include "SignalManager.h"
 
+class KURL;
 class LabelList;
 class LabelType;
 class MouseMark;
@@ -86,14 +88,11 @@ public:
     double samples2ms(unsigned int samples);
 
     /**
-     * Closes the current signal and loads a new one
-     * from a file.
-     * @param filename name of the .wav or .asc file
-     * @param type one of WAV or ASCII
+     * Closes the current signal and loads a new one from a file.
+     * @param url URL of the file to be loaded
      * @return 0 if succeeded or error code < 0
-     * @todo use QUrl instead of QString
      */
-    int loadFile(const QString  &filename, int type);
+    int loadFile(const KURL &url);
 
     /**
      * Forwards information for creation of a new signal to the
@@ -104,15 +103,11 @@ public:
                    unsigned int bits, unsigned int tracks);
 
     /** @todo (re)implementation */
-    int saveFile(const QString &filename, unsigned int bits,
-                 int type, bool selection = false);
+    int saveFile(const KURL &url, unsigned int bits, bool selection = false);
 
     /** @todo (re)implementation */
     void saveBlocks (int);
 
-    /** @todo (re)implementation */
-    void setSignal (SignalManager *signal);
-
     /**
      * Closes the current signal
      */
@@ -126,12 +121,6 @@ public:
      */
     void setZoom(double new_zoom);
 
-    /**
-     * Returns a QBitmap with an overview of all currently present
-     * signals.
-     */
-    QBitmap *overview(unsigned int width, unsigned int height);
-
     /**
      * Checks if a pixel position is near to the left or right border
      * of a selection. The tolerance is 2% of the currently
@@ -205,6 +194,9 @@ public slots:
      */
     inline double zoom() { return m_zoom; };
 
+    /** Returns the width of the current view in samples */
+    int displaySamples();
+
     /**
      * Zooms into the selected range between the left and right marker.
      */
@@ -487,12 +479,6 @@ private:
      */
     void refreshLayer(int layer);
 
-    /**
-     * (re)starts the timer that is responsible for redrawing
-     * the playpointer if we are in playback mode.
-     */
-    void playback_startTimer();
-
     /**
      * Converts a sample index into a pixel offset using the current zoom
      * value. Always rounds up or downwards. If the number of pixels or the
@@ -566,9 +552,6 @@ private:
     /** our signal manager */
     SignalManager m_signal_manager;
 
-    /** timer for delayed screen refreshes (don't refresh too often) */
-    QTimer m_refresh_timer;
-
     /** list of track pixmaps */
     QList<TrackPixmap> m_track_pixmaps;
 
