@@ -19,6 +19,7 @@
 #include "config.h"
 
 #include <errno.h>
+#include <math.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -34,25 +35,29 @@
 
 #include <kcombobox.h>
 #include <kfiledialog.h>
+#include <kfilefilter.h>
+#include <kiconloader.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kmenubar.h>
+#include <kstatusbar.h>
 #include <kstddirs.h>
 #include <ktoolbarbutton.h>
 
-#include "libkwave/FileFormat.h"
 #include "libkwave/FileLoader.h"
 #include "libkwave/LineParser.h"
 #include "libkwave/Parser.h"
 
 #include "libgui/Dialog.h"
 #include "libgui/MenuManager.h"
+#include "libgui/KwaveFileDialog.h"
 #include "libgui/KwavePlugin.h" // for some helper functions
 
 #include "mt/ThreadsafeX11Guard.h"
 
 #include "KwaveApp.h"
 #include "ClipBoard.h"
+#include "CodecManager.h"
 #include "MainWidget.h"
 #include "TopWidget.h"
 #include "PlaybackController.h"
@@ -70,11 +75,11 @@
 //#include "toolbar/eraser.xpm"
 //#include "toolbar/delete.xpm"
 
-#include "toolbar/play.xpm"
-#include "toolbar/loop.xpm"
-#include "toolbar/pause.xpm"
-#include "toolbar/pause2.xpm"
-#include "toolbar/stop.xpm"
+#include "pics/playback_loop.xpm"
+#include "pics/playback_pause.xpm"
+#include "pics/playback_pause2.xpm"
+#include "pics/playback_start.xpm"
+#include "pics/playback_stop.xpm"
 
 #include "toolbar/zoomrange.xpm"
 #include "toolbar/zoomin.xpm"
@@ -82,6 +87,12 @@
 #include "toolbar/zoomnormal.xpm"
 #include "toolbar/zoomall.xpm"
 
+#ifdef HAVE_KDEVERSION_H
+#include <kdeversion.h>
+#else
+#define KDE_VERSION_MAJOR 2
+#endif
+
 #ifndef min
 #define min(x,y) (( (x) < (y) ) ? (x) : (y) )
 #endif
@@ -105,25 +116,37 @@
 //***************************************************************************
 //***************************************************************************
 TopWidget::ZoomListPrivate::ZoomListPrivate()
-    :QStringList()
+    :QStringList(), m_milliseconds()
 {
-    clear();
-    append("800 %");
-    append("400 %");
-    append("200 %");
-    append("100 %");
-    append("33 %");
-    append("10 %");
-    append("3 %");
-    append("1 %");
-    append("0.1 %");
-    append("0.01 %");
+    append("1 ms",              1L);
+    append("10 ms",            10L);
+    append("100 ms",          100L);
+    append("1 sec",          1000L);
+    append("10 sec",     10L*1000L);
+    append("30 sec",     30L*1000L);
+    append("1 min",   1L*60L*1000L);
+    append("10 min", 10L*60L*1000L);
+    append("30 min", 30L*60L*1000L);
+    append("60 min", 60L*60L*1000L);
 };
 
+//***************************************************************************
+void TopWidget::ZoomListPrivate::append(const char *text, unsigned int ms)
+{
+    QStringList::append(text);
+    m_milliseconds.append(ms);
+}
+
+//***************************************************************************
+unsigned int TopWidget::ZoomListPrivate::ms(int index)
+{
+    return m_milliseconds[index];
+}
+
+//***************************************************************************
 //***************************************************************************
 TopWidget::TopWidget(KwaveApp &main_app)
-    :KMainWindow(),
-    m_app(main_app)
+    :KMainWindow(), m_app(main_app), m_url()
 {
     int id=1000; // id of toolbar items
     KIconLoader icon_loader;
@@ -141,7 +164,6 @@ TopWidget::TopWidget(KwaveApp &main_app)
     m_main_widget = 0;
     m_menu_manager = 0;
     m_pause_timer = 0;
-    m_filename = "";
     m_toolbar = 0;
     m_zoomselect = 0;
 
@@ -179,13 +201,6 @@ TopWidget::TopWidget(KwaveApp &main_app)
     status_bar->insertItem("", STATUS_ID_SELECTED);
     setStatusInfo(SAMPLE_MAX,99,196000,24); // affects the menu !
 
-//    //enable drop of local files onto kwave window
-//    dropZone = new KDNDDropZone( this , DndURL);
-//    ASSERT(dropZone);
-//    if (!dropZone) return;
-//    connect( dropZone, SIGNAL( dropAction( KDNDDropZone *)),
-//	     this, SLOT( dropEvent( KDNDDropZone *)));
-
     // load the menu from file
     QString menufile = locate("data", "kwave/menus.config");
     FileLoader loader(menufile);
@@ -393,9 +408,16 @@ TopWidget::TopWidget(KwaveApp &main_app)
 	m_main_widget, SLOT(zoomAll()), true,
 	i18n("zoom to all"));
     m_id_zoomall = id++;
-	
+
+#if KDE_VERSION_MAJOR >= 3
+    // add a dummy placeholder, otherwise the minimum size will be wrong
+    QStringList factors(m_zoom_factors);
+    factors.append(" 99:99 min ");
+#else
+    QStringList &factors = m_zoom_factors;
+#endif
     m_id_zoomselect = id++;
-    m_toolbar->insertCombo(m_zoom_factors, m_id_zoomselect,
+    m_toolbar->insertCombo(factors, m_id_zoomselect,
 	true, SIGNAL(activated(int)),
 	this, SLOT(selectZoom(int)), true,
 	i18n("select zoom factor"));
@@ -405,29 +427,33 @@ TopWidget::TopWidget(KwaveApp &main_app)
     ASSERT(m_zoomselect);
     if (!m_zoomselect) return;
 
+#if KDE_VERSION_MAJOR < 3
     m_zoomselect->adjustSize();
     int h = m_zoomselect->sizeHint().height();
     m_zoomselect->setFixedHeight(h);
     m_zoomselect->setMinimumWidth(
-        max(m_zoomselect->sizeHint().width()+10, 3*h));
+        max(m_zoomselect->sizeHint().width()+20,3*h));
     m_zoomselect->setAutoResize(false);
+#else
+    int h = m_zoomselect->sizeHint().height();
+#endif
     m_zoomselect->setFocusPolicy(QWidget::NoFocus);
+
     m_toolbar->setMinimumHeight(max(m_zoomselect->sizeHint().height()+2,
 	m_toolbar->sizeHint().height()));
-
     m_toolbar->insertSeparator(-1);
     updateToolbar();
 
     // connect the playback controller
     connect(&(m_main_widget->playbackController()),
             SIGNAL(sigPlaybackStarted()),
-            this, SLOT(updateToolbar()));
+            this, SLOT(updatePlaybackControls()));
     connect(&(m_main_widget->playbackController()),
             SIGNAL(sigPlaybackPaused()),
             this, SLOT(playbackPaused()));
     connect(&(m_main_widget->playbackController()),
             SIGNAL(sigPlaybackStopped()),
-            this, SLOT(updateToolbar()));
+            this, SLOT(updatePlaybackControls()));
 
     // connect the signal manager
     SignalManager *signal_manager = &(m_main_widget->signalManager());
@@ -491,6 +517,7 @@ TopWidget::TopWidget(KwaveApp &main_app)
 	    "<a href=http://www.arts-project.org>"\
 	    "http://www.arts-project.org</a>."
 	));
+	qApp->exit();
 	return;
     };
 
@@ -501,6 +528,11 @@ TopWidget::TopWidget(KwaveApp &main_app)
 
     setTrackInfo(0);
     updateMenu();
+
+#if KDE_VERSION_MAJOR >= 3
+    // layout is finished, remove dummy/placeholder zoom entry
+    m_zoomselect->removeItem(m_zoomselect->count()-1);
+#endif
 }
 
 //***************************************************************************
@@ -538,20 +570,52 @@ TopWidget::~TopWidget()
     if (m_menu_manager) delete m_menu_manager;
     m_menu_manager=0;
 
-    if (m_app.closeWindow(this)) m_app.exit();
+    m_app.closeWindow(this);
 }
 
 //***************************************************************************
-void TopWidget::executeCommand(const QString &command)
+int TopWidget::executeCommand(const QString &line)
 {
-//    debug("TopWidget::executeCommand(%s)", command.data()); // ###
-    if (!command.length()) return;
-    if (command.stripWhiteSpace().startsWith("#")) return; // only a comment
+    int result = 0;
+    bool use_recorder = true;
+    QString command = line;
 
+//    debug("TopWidget::executeCommand(%s)", command.data()); // ###
+    if (!command.length()) return 0; // empty line -> nothing to do
+    if (command.stripWhiteSpace().startsWith("#")) return 0; // only a comment
+
+    // special case: if the command contains ";" it is a list of
+    // commands -> macro !
+    Parser parse_list(command);
+    if (parse_list.hasMultipleCommands()) {
+	QStringList macro = parse_list.commandList();
+	for (QStringList::Iterator it=macro.begin(); it!=macro.end(); ++it) {
+	    result = executeCommand("nomacro:"+(*it));
+	    ASSERT(!result);
+	    if (result) {
+		warning("macro execution of '%s' failed: %d",
+		        (*it).data(), result);
+		return result; // macro failed :-(
+	    }
+	
+	    // wait until the command has completed !
+	    ASSERT(m_plugin_manager);
+	    if (m_plugin_manager) m_plugin_manager->sync();
+	}
+	return result;
+    }
+
+    // check if the macro recorder has to be disabled for this command
+    if (command.startsWith("nomacro:")) {
+	use_recorder = false;
+	command = command.mid(QString("nomacro:").length());
+    }
+
+    // parse one single command
     Parser parser(command);
 
     if (m_app.executeCommand(command)) {
-	return ;
+	return 0;
     CASE_COMMAND("plugin")
 	QString name = parser.firstParam();
 	QStringList *params = 0;
@@ -570,7 +634,8 @@ void TopWidget::executeCommand(const QString &command)
 	debug("TopWidget::executeCommand(): with %d parameter(s)",
 		(params) ? params->count() : 0);
 	ASSERT(m_plugin_manager);
-	if (m_plugin_manager) m_plugin_manager->executePlugin(name, params);
+	if (m_plugin_manager)
+	    result = m_plugin_manager->executePlugin(name, params);
     CASE_COMMAND("plugin:execute")
 	QStringList params;
 	int cnt = parser.count();
@@ -579,72 +644,93 @@ void TopWidget::executeCommand(const QString &command)
 	    params.append(parser.nextParam());
 	}
 	ASSERT(m_plugin_manager);
-	if (m_plugin_manager) m_plugin_manager->executePlugin(
-            name, &params);
+	result = (m_plugin_manager) ?
+	          m_plugin_manager->executePlugin(name, &params) : -ENOMEM;
     CASE_COMMAND("plugin:setup")
 	QString name(parser.firstParam());
 	ASSERT(m_plugin_manager);
-	if (m_plugin_manager) m_plugin_manager->setupPlugin(name);
+	if (m_plugin_manager) result = m_plugin_manager->setupPlugin(name);
     CASE_COMMAND("menu")
 	ASSERT(m_menu_manager);
-	if (m_menu_manager) m_menu_manager->executeCommand(command);
+	if (m_menu_manager) /*result = */m_menu_manager->executeCommand(command);
     CASE_COMMAND("newsignal");
 	unsigned int samples = parser.toUInt();
 	double       rate    = parser.toDouble();
 	unsigned int bits    = parser.toUInt();
 	unsigned int tracks  = parser.toUInt();
-	newSignal(samples, rate, bits, tracks);
+	result = newSignal(samples, rate, bits, tracks);
     CASE_COMMAND("open")
 	QString filename = parser.nextParam();
 	if (!filename.isEmpty()) {
 	    // open the selected file
-	    int len = filename.length();
-	    if (filename.lower().findRev(".wav") == len-4) {
-		loadFile(filename, WAV);
-	    } else if (filename.lower().findRev(".asc") == len-4) {
-		loadFile(filename, ASCII);
-	    }
+	    result = loadFile(filename);
 	} else {
 	    // show file open dialog
-	    openFile();
+	    result = openFile();
 	}
     CASE_COMMAND("openrecent")
-	openRecent(command);
+	result = openRecent(command);
+    CASE_COMMAND("playback")
+	result = executePlaybackCommand(parser.firstParam()) ? 0 : -1;
     CASE_COMMAND("save")
-	saveFile();
+	result = saveFile();
     CASE_COMMAND("close")
-	closeFile();
+	result = closeFile();
     CASE_COMMAND("resolution")
-	resolution(command);
+	result = resolution(command);
     CASE_COMMAND("revert")
-	revert();
-    CASE_COMMAND("importascii")
-	importAsciiFile();
-    CASE_COMMAND("exportascii")
-	exportAsciiFile();
+	result = revert();
     CASE_COMMAND("saveas")
-	saveFileAs(false);
+	result = saveFileAs(false);
     CASE_COMMAND("loadbatch")
-	loadBatch(command);
+	result = loadBatch(command);
     CASE_COMMAND("saveselect")
-	saveFileAs(true);
+	result = saveFileAs(true);
     CASE_COMMAND("quit")
-	close();
+	result = close();
     } else {
 	ASSERT(m_main_widget);
-	if ((m_main_widget) && (m_main_widget->executeCommand(command))) {
-	    return ;
-	}
+	result = (m_main_widget &&
+	    m_main_widget->executeCommand(command)) ? 0 : -1;
     }
 
+    if (use_recorder) {
+	// append the command to the macro recorder
+        // @TODO macro recording...
+    }
+
+    return result;
 }
 
 //***************************************************************************
-void TopWidget::loadBatch(const QString &str)
+int TopWidget::loadBatch(const QString &str)
 {
     Parser parser(str);
     FileLoader loader(parser.firstParam());
-    parseCommands(loader.buffer());
+    return parseCommands(loader.buffer());
+}
+
+//***************************************************************************
+int TopWidget::executePlaybackCommand(const QString &command)
+{
+    ASSERT(m_main_widget);
+    if (!m_main_widget) return -1;
+
+    PlaybackController &controller = m_main_widget->playbackController();
+    if (command == "start") {
+	controller.playbackStart();
+    } else if (command == "loop") {
+	controller.playbackLoop();
+    } else if (command == "stop") {
+	controller.playbackStop();
+    } else if (command == "pause") {
+	pausePressed();
+    } else if (command == "continue") {
+	pausePressed();
+    } else {
+	return -EINVAL;
+    }
+    return 0;
 }
 
 //***************************************************************************
@@ -655,33 +741,42 @@ SignalManager &TopWidget::signalManager()
 }
 
 //***************************************************************************
-void TopWidget::parseCommands(const QByteArray &buffer)
+int TopWidget::parseCommands(const QByteArray &buffer)
 {
+    int result = 0;
+
     LineParser lineparser(buffer);
     QString line = lineparser.nextLine();
-    while (line.length()) {
-	executeCommand(line);
+    while (line.length() && !result) {
+	result = executeCommand(line);
 	line = lineparser.nextLine();
     }
+    return result;
 }
 
 //***************************************************************************
-void TopWidget::revert()
+int TopWidget::revert()
 {
-    QString name = m_filename;
-    loadFile(name, WAV);
+    ASSERT(m_url.isValid() && !m_url.isMalformed());
+    if (!m_url.isValid() || m_url.isMalformed()) return -EINVAL;
+
+    KURL url(m_url);
+    return loadFile(url);
 }
 
 //***************************************************************************
-void TopWidget::resolution(const QString &str)
+int TopWidget::resolution(const QString &str)
 {
     Parser parser (str);
     int bps = parser.toInt();
 
     if ( (bps >= 0) && (bps <= 24) && (bps % 8 == 0)) {
 	m_save_bits = bps;
+	return 0;
+    } else {
+	warning("out of range");
+	return -EINVAL;
     }
-    else warning("out of range");
 }
 
 //***************************************************************************
@@ -704,10 +799,10 @@ bool TopWidget::closeFile()
 
     if (m_main_widget) m_main_widget->closeSignal();
 
-    m_filename = "";
+    m_url = KURL();
     updateCaption();
     m_zoomselect->clearEdit();
-    emit sigSignalNameChanged(m_filename);
+    emit sigSignalNameChanged(signalName());
 
     updateMenu();
     updateToolbar();
@@ -717,30 +812,30 @@ bool TopWidget::closeFile()
 }
 
 //***************************************************************************
-int TopWidget::loadFile(const QString &filename, int type)
+int TopWidget::loadFile(const KURL &url)
 {
     ASSERT(m_main_widget);
     if (!m_main_widget) return -1;
 
-    // abort if new file not valid
-    ASSERT(filename.length());
-    if (!filename.length()) return -1;
+    // abort if new file not valid and local
+    ASSERT(url.isLocalFile());
+    if (!url.isLocalFile()) return -1;
 
     // try to close the previous file
     if (!closeFile()) return -1;
 
-    m_filename = filename;
-    emit sigSignalNameChanged(m_filename);
+    emit sigSignalNameChanged(url.path());
 
-    if (!m_main_widget->loadFile(filename, type)) {
+    if (!m_main_widget->loadFile(url)) {
 	// succeeded
+	m_url = url;
 	updateCaption();
 	m_save_bits = m_main_widget->bits();
     } else {
 	// load failed
 	closeFile();
     }
-    m_app.addRecentFile(m_filename);
+    m_app.addRecentFile(signalName());
     updateMenu();
     updateToolbar();
 
@@ -748,35 +843,22 @@ int TopWidget::loadFile(const QString &filename, int type)
 }
 
 //***************************************************************************
-void TopWidget::openRecent(const QString &str)
+int TopWidget::openRecent(const QString &str)
 {
     Parser parser(str);
-    loadFile(parser.firstParam(), WAV);
+    return loadFile(parser.firstParam());
 }
 
 //***************************************************************************
-void TopWidget::openFile()
+int TopWidget::openFile()
 {
-    QString filename = KFileDialog::getOpenFileName(
-	":kwave-open-dir", "*.wav", this);
-    if (filename.length()) loadFile(filename, WAV);
-}
-
-//***************************************************************************
-void TopWidget::importAsciiFile()
-{
-    QString filename = KFileDialog::getOpenFileName(
-	":kwave-open-dir", "*.asc", this);
-    if (filename.length()) loadFile(filename, ASCII);
-}
-
-//***************************************************************************
-void TopWidget::exportAsciiFile()
-{
-    QString filename = KFileDialog::getOpenFileName(
-	":kwave-open-dir", "*.asc", this);
-    if (filename.length()) m_main_widget->saveFile(filename,
-    	m_save_bits, ASCII, false);
+    KwaveFileDialog dlg(":<kwave_open_dir>", CodecManager::decodingFilter(),
+        this, "Kwave open file", true);
+    dlg.setMode(static_cast<KFile::Mode>(KFile::File | KFile::ExistingOnly));
+    dlg.setOperationMode(KFileDialog::Opening);
+    dlg.setCaption(i18n("Open"));
+    if (dlg.exec() == QDialog::Accepted) return loadFile(dlg.selectedURL());
+    return -1;
 }
 
 //***************************************************************************
@@ -786,8 +868,8 @@ int TopWidget::saveFile()
     ASSERT(m_main_widget);
     if (!m_main_widget) return -EINVAL;
 
-    if (m_filename.length() && (m_filename != NEW_FILENAME)) {
-	res = m_main_widget->saveFile(m_filename, m_save_bits, 0, false);
+    if (signalName() != NEW_FILENAME) {
+	res = m_main_widget->saveFile(m_url, m_save_bits, false);
     } else res = saveFileAs(false);
 
     updateCaption();
@@ -803,23 +885,33 @@ int TopWidget::saveFileAs(bool selection)
     ASSERT(m_main_widget);
     if (!m_main_widget) return -EINVAL;
 
-    QString name = KFileDialog::getSaveFileName(":kwave-savedir",
-	"*.wav", m_main_widget);
+    KwaveFileDialog dlg(":<kwave_save_dir>", CodecManager::encodingFilter(),
+        this, "Kwave save file", true);
+    dlg.setKeepLocation(true);
+//    dlg.setMode(static_cast<KFile::Mode>(KFile::));
+    dlg.setOperationMode(KFileDialog::Saving);
+    dlg.setCaption(i18n("Save As"));
+    if (dlg.exec() != QDialog::Accepted) return -1;
 
-    if (!name.isNull()) {
+    KURL url = dlg.selectedURL();
+//    getSaveURL(":<kwave-savedir>",
+//	CodecManager::encodingFilter(), m_main_widget,
+//	"Kwave open file", true);
+
+    if (!url.isEmpty()) {
+	QString name = url.path();
 	QFileInfo path(name);
 	
-	// add the extension .wav if necessary
-	if ((path.extension(false) != "wav") &&
-	    (path.extension(false) != "WAV"))
-	{
+	// add the (default) extension .wav if necessary
+	if (path.extension(false) == "") {
 	    name += ".wav";
 	    path = name;
+	    url.setPath(name);
 	}
 	
 	// check if the file exists and ask before overwriting it
 	// if it is not the old filename
-	if ((m_filename != name) && (path.exists())) {
+	if ((name != signalName()) && (path.exists())) {
 	    if (KMessageBox::warningYesNo(this,
 	        i18n("The file '%1' already exists. Do you really "\
 	        "want to overwrite it?").arg(name)) != KMessageBox::Yes)
@@ -828,11 +920,11 @@ int TopWidget::saveFileAs(bool selection)
 	    }
 	}
 	
-	m_filename = name;
-	res = m_main_widget->saveFile(m_filename, m_save_bits, 0, selection);
+	m_url = url;
+	res = m_main_widget->saveFile(m_url, m_save_bits, selection);
 	
 	updateCaption();
-	m_app.addRecentFile(m_filename);
+	m_app.addRecentFile(signalName());
 	updateMenu();
     }
 
@@ -840,14 +932,14 @@ int TopWidget::saveFileAs(bool selection)
 }
 
 //***************************************************************************
-void TopWidget::newSignal(unsigned int samples, double rate,
-                          unsigned int bits, unsigned int tracks)
+int TopWidget::newSignal(unsigned int samples, double rate,
+                         unsigned int bits, unsigned int tracks)
 {
     // abort if the user pressed cancel
-    if (!closeFile()) return;
+    if (!closeFile()) return -1;
 
-    m_filename = NEW_FILENAME;
-    emit sigSignalNameChanged(m_filename);
+    m_url = "file:"+NEW_FILENAME;
+    emit sigSignalNameChanged(signalName());
 
     m_main_widget->newSignal(samples, rate, bits, tracks);
 
@@ -855,12 +947,15 @@ void TopWidget::newSignal(unsigned int samples, double rate,
     m_save_bits = bits;
     updateMenu();
     updateToolbar();
+
+    return 0;
 }
 
 //***************************************************************************
-const QString &TopWidget::getSignalName()
+QString TopWidget::signalName()
 {
-    return m_filename;
+    return (m_url.isValid() && !m_url.isMalformed()) ?
+            m_url.path() : QString("");
 }
 
 //***************************************************************************
@@ -873,11 +968,12 @@ void TopWidget::selectZoom(int index)
     if ((unsigned int)index >= m_zoom_factors.count())
 	index = m_zoom_factors.count()-1;
 
-    double new_zoom;
-    QStringList::Iterator text = m_zoom_factors.at(index);
-    new_zoom = (text != 0) ? (*text).toDouble() : 0.0;
-    if (new_zoom != 0.0) new_zoom = (100.0 / new_zoom);
-
+    const double rate = signalManager().rate();
+    const double ms = m_zoom_factors.ms(index);
+    unsigned int width = m_main_widget->displayWidth();
+    ASSERT(width > 1);
+    if (width <= 1) width = 2;
+    const double new_zoom = rint(((rate*ms)/1E3)-1) / (double)(width-1);
     m_main_widget->setZoom(new_zoom);
 
     // force the zoom factor to be set, maybe the current selection
@@ -895,11 +991,29 @@ void TopWidget::setZoomInfo(double zoom)
     if (zoom <= 0.0) return; // makes no sense or signal is empty
     if (!m_zoomselect) return;
 
-    double percent = (double)100.0 / zoom;
     QString strZoom;
-
     if ((m_main_widget) && (m_main_widget->tracks())) {
-	strZoom = KwavePlugin::zoom2string(percent);
+	unsigned int rate = signalManager().rate();
+	if (rate) {
+	    // time display mode
+	    double ms = m_main_widget->displaySamples()*1E3/(double)rate;
+	    int s = (int)floor(ms / 1000.0);
+	    int m = (int)floor(s / 60.0);
+	
+	    if (m >= 1) {
+		strZoom = strZoom.sprintf("%02d:%02d min", m, s % 60);
+	    } else if (s >= 1) {
+		strZoom = strZoom.sprintf("%d sec", s);
+	    } else if (ms >= 1) {
+		strZoom = strZoom.sprintf("%d ms", (int)round(ms));
+	    } else if (ms >= 0.01) {
+		strZoom = strZoom.sprintf("%0.3g ms", ms);
+	    }
+	} else {
+	    // percent mode
+	    double percent = (double)100.0 / zoom;
+	    strZoom = KwavePlugin::zoom2string(percent);
+	}
     }
 
     (strZoom.length()) ? m_zoomselect->setEditText(strZoom) :
@@ -927,8 +1041,8 @@ void TopWidget::setStatusInfo(unsigned int length, unsigned int /*tracks*/,
 
     // sample rate and resolution
     if (bits) {
-	txt = " "+i18n("Mode: %u bit@%0.3f kHz")+" ";
-	txt = txt.sprintf(txt, bits, (double)rate *1E-3);
+	txt = " "+i18n("Mode: %0.3f kHz@%u bit")+" ";
+	txt = txt.sprintf(txt, (double)rate *1E-3, bits);
     } else txt = "";
     statusBar()->changeItem(txt, STATUS_ID_MODE);
 
@@ -951,7 +1065,6 @@ void TopWidget::setTrackInfo(unsigned int tracks)
     // enable/disable all items that depend on having a signal
     bool have_signal = (tracks != 0);
     m_menu_manager->setItemEnabled("@SIGNAL", have_signal);
-
 }
 
 //***************************************************************************
@@ -1063,8 +1176,8 @@ void TopWidget::updateMenu()
     QString bps = QString("ID_FILE_SAVE_RESOLUTION_%1").arg(m_save_bits);
     m_menu_manager->selectItem("@BITS_PER_SAMPLE", bps);
 
-    // enable/disable all items that depend on having a filel
-    bool have_file = (m_filename.length() != 0);
+    // enable/disable all items that depend on having a file
+    bool have_file = (signalName().length() != 0);
     m_menu_manager->setItemEnabled("@NOT_CLOSED", have_file);
 }
 
@@ -1076,6 +1189,26 @@ void TopWidget::updateToolbar()
     if (!m_toolbar) return;
     if (!m_main_widget) return;
 
+    bool have_signal = m_main_widget->tracks();
+
+    m_toolbar->setItemEnabled(m_id_zoomselection, have_signal);
+    m_toolbar->setItemEnabled(m_id_zoomin, have_signal);
+    m_toolbar->setItemEnabled(m_id_zoomout, have_signal);
+    m_toolbar->setItemEnabled(m_id_zoomnormal, have_signal);
+    m_toolbar->setItemEnabled(m_id_zoomall, have_signal);
+    m_toolbar->setItemEnabled(m_id_zoomselect, have_signal);
+
+    updatePlaybackControls();
+}
+
+//***************************************************************************
+void TopWidget::updatePlaybackControls()
+{
+    ASSERT(m_toolbar);
+    ASSERT(m_main_widget);
+    if (!m_toolbar) return;
+    if (!m_main_widget) return;
+
     bool have_signal = m_main_widget->tracks();
     bool playing = m_main_widget->playbackController().running();
     bool paused  = m_main_widget->playbackController().paused();
@@ -1084,7 +1217,16 @@ void TopWidget::updateToolbar()
 	m_pause_timer->stop();
 	delete m_pause_timer;
 	m_pause_timer = 0;
-	m_toolbar->setButtonPixmap(m_id_pause, xpm_pause);
+	
+	// NOTE: working with toolbar->getButton is ugly and NOT
+	//       recommended, but the only way this works in KDE3 :-(
+	KToolBarButton *button = m_toolbar->getButton(m_id_pause);
+	ASSERT(button);
+	if (!button) return;
+	button->setDefaultPixmap(xpm_pause);
+	
+	// doesn't work with the damned buggy KDE3
+	// m_toolbar->setButtonPixmap(m_id_pause, xpm_pause);
     }
 
     // enable/disable the buttons
@@ -1094,12 +1236,17 @@ void TopWidget::updateToolbar()
     m_toolbar->setItemEnabled(m_id_pause, have_signal && (playing || paused));
     m_toolbar->setItemEnabled(m_id_stop,  have_signal && (playing || paused));
 
-    m_toolbar->setItemEnabled(m_id_zoomselection, have_signal);
-    m_toolbar->setItemEnabled(m_id_zoomin, have_signal);
-    m_toolbar->setItemEnabled(m_id_zoomout, have_signal);
-    m_toolbar->setItemEnabled(m_id_zoomnormal, have_signal);
-    m_toolbar->setItemEnabled(m_id_zoomall, have_signal);
-    m_toolbar->setItemEnabled(m_id_zoomselect, have_signal);
+    m_menu_manager->setItemEnabled("ID_PLAYBACK_START",
+	have_signal && !playing);
+    m_menu_manager->setItemEnabled("ID_PLAYBACK_LOOP",
+	have_signal && !playing);
+    m_menu_manager->setItemEnabled("ID_PLAYBACK_PAUSE",
+	have_signal && (playing));
+    m_menu_manager->setItemEnabled("ID_PLAYBACK_CONTINUE",
+	have_signal && (paused));
+    m_menu_manager->setItemEnabled("ID_PLAYBACK_STOP",
+	have_signal && (playing || paused));
+
 }
 
 //***************************************************************************
@@ -1126,8 +1273,18 @@ void TopWidget::blinkPause()
     ASSERT(m_toolbar);
     if (!m_toolbar) return;
 
-    m_toolbar->setButtonPixmap(m_id_pause,
-	m_blink_on ? xpm_pause2 : xpm_pause);
+    // NOTE: working with toolbar->getButton is ugly and NOT
+    //       recommended, but the only way this works in KDE3 :-(
+    KToolBarButton *button = m_toolbar->getButton(m_id_pause);
+    ASSERT(button);
+    if (!button) return;
+    button->setDefaultPixmap(m_blink_on ? xpm_pause2 : xpm_pause);
+
+    // this would be the correct way, but KDE-3.0.1 is too buggy
+    // to let it work...
+    //    m_toolbar->setButtonPixmap(m_id_pause,
+    //        m_blink_on ? xpm_pause2 : xpm_pause);
+
     m_blink_on = !m_blink_on;
 }
 
@@ -1162,15 +1319,15 @@ void TopWidget::updateCaption()
     bool modified = signalManager().isModified();
 
     // shortcut if no file loaded
-    if (m_filename.length() == 0) {
+    if (signalName().length() == 0) {
 	setCaption(0);
 	return;
     }
 
     if (modified)
-	setCaption("* "+m_filename+i18n(" (modified)"));
+	setCaption("* "+signalName()+i18n(" (modified)"));
     else
-	setCaption(m_filename);
+	setCaption(signalName());
 }
 
 //***************************************************************************
