@@ -19,9 +19,11 @@
 #ifndef _TOP_WIDGET_H_
 #define _TOP_WIDGET_H_
 
+#include "config.h"
 #include <qcstring.h>
 #include <qstringlist.h>
 #include <kmainwindow.h>
+#include <kurl.h>
 
 class QCloseEvent;
 class QTimer;
@@ -68,11 +70,10 @@ public:
     /**
      * Loads a new file and updates the widget's title, menu, status bar
      * and so on.
-     * @param filename path to the file to be loaded
-     * @param type format of the file (WAV or ASCII)
+     * @param url URL of the file to be loaded
      * @return 0 if successful
      */
-    int loadFile(const QString &filename, int type = 0);
+    int loadFile(const KURL &url);
 
     /**
      * Returns the reference to the Kwave application
@@ -83,7 +84,7 @@ public:
      * Returns a reference to the current name of the signal. If no signal is
      * loaded the string is zero-length.
      */
-    const QString &getSignalName();
+    QString signalName();
 
     /**
      * Parses a buffer (that is intended to contain the content
@@ -91,14 +92,14 @@ public:
      * parsed commands.
      * @param buffer the list of commands
      */
-    void parseCommands(const QByteArray &buffer);
+    int parseCommands(const QByteArray &buffer);
 
     /**
      * Loads a batch file into memory, parses and executes
      * all commands in it.
      * @param filename name of the batch file
      */
-    void loadBatch(const QString &filename);
+    int loadBatch(const QString &filename);
 
     /**
      * Returns a pointer to the current signal manager.
@@ -107,7 +108,7 @@ public:
 
 public slots:
 
-    void executeCommand(const QString &command);
+    int executeCommand(const QString &command);
 
     /**
      * Updates the list of recent files in the menu, maybe some other
@@ -174,6 +175,9 @@ private slots:
     /** updates all elements in the toolbar */
     void updateToolbar();
 
+    /** updates only the playback controls */
+    void updatePlaybackControls();
+
     /** called if the playback has been paused */
     void playbackPaused();
 
@@ -236,13 +240,13 @@ protected:
      * Discards all changes to the current file and loads
      * it again.
      */
-    void revert();
+    int revert();
 
     /**
      * Shows an "open file" dialog and opens the .wav file the
      * user has selected.
      */
-    void openFile();
+    int openFile();
 
     /**
      * Closes the current file and updates the menu and other controls.
@@ -253,12 +257,6 @@ protected:
      */
     bool closeFile();
 
-    /**
-     * Like openFile(), but reads in ASCII format.
-     * @see #openFile()
-     */
-    void importAsciiFile();
-
     /** Saves the current file. */
     int saveFile();
 
@@ -276,28 +274,27 @@ protected:
      * @param bits number of bits per sample
      * @param tracks number of tracks
      */
-    void newSignal(unsigned int samples, double rate,
-                   unsigned int bits, unsigned int tracks);
-
-    /**
-     * Like saveFileAs(), but saves in ASCII format. */
-    void exportAsciiFile();
+    int newSignal(unsigned int samples, double rate,
+                  unsigned int bits, unsigned int tracks);
 
     /**
      * Opens a file contained in the list of recent files.
      * @param str the entry contained in the list
      */
-    void openRecent (const QString &str);
+    int openRecent(const QString &str);
 
     /**
      * Sets a new resolution for saving in bits per sample.
      * @param str str the resolution in string representation
      */
-    void resolution (const QString &str);
+    int resolution(const QString &str);
 
     /** Updates the caption with the filename */
     void updateCaption();
 
+    /** handle playback commands, like play/stop/pause etc... */
+    int executePlaybackCommand(const QString &command);
+
 private:
 
     /**
@@ -306,9 +303,14 @@ private:
      */
     class ZoomListPrivate: public QStringList
     {
-	public:
+    public:
 	ZoomListPrivate();
 	virtual ~ZoomListPrivate() {};
+	unsigned int ms(int index);
+    private:
+	void append(const char *text, unsigned int ms);
+    private:
+	QValueList<unsigned int> m_milliseconds;
     };
 
     /** Initialized list of zoom factors */
@@ -335,8 +337,8 @@ private:
      */
     KToolBar *m_toolbar;
 
-    /** Name of the current signal or file. Zero-Length if nothing loaded */
-    QString m_filename;
+    /** URL of the current signal or file. Empty if nothing loaded */
+    KURL m_url;
 
     /** menu manager for this window */
     MenuManager *m_menu_manager;
