@@ -16,22 +16,13 @@
  ***************************************************************************/
 
 #include "config.h"
-#include "math.h"
-
-#include <qbitmap.h>
 #include <qpainter.h>
 #include <qpixmap.h>
-#include <qtimer.h>
-
-#include "mt/Mutex.h"
-#include "mt/MutexGuard.h"
-#include "kwave/SignalManager.h"
 
 #include "OverViewWidget.h"
 
 #define TIMER_INTERVAL 100        /* ms */
 #define MIN_SLIDER_WIDTH m_height /* pixel */
-#define CACHE_SIZE 8192           /* number of cache entries */
 
 #ifndef min
 #define min(x,y) (( (x) < (y) ) ? (x) : (y) )
@@ -48,44 +39,19 @@
 
 //*****************************************************************************
 OverViewWidget::OverViewWidget(SignalManager &signal, QWidget *parent)
-    :QWidget(parent), m_signal(signal), m_min(), m_max(), m_state(),
-     m_count(0), m_scale(1), m_lock()
+    :QWidget(parent), m_width(0), m_height(0), m_grabbed(0), m_mouse_pos(0),
+     m_slider_width(0), m_view_width(0), m_view_length(0),m_view_offset(0),
+     m_dir(0), m_redraw(false), m_timer(), m_bitmap(), m_pixmap(0),
+     m_cache(signal)
 {
-    m_bitmap = 0;
-    m_dir = 0;
-    m_grabbed = 0;
-    m_height = 0;
-    m_mouse_pos = 0;
-    m_pixmap = 0;
-    m_redraw = false;
-    m_slider_width = 0;
-    m_view_length = 0;
-    m_view_offset = 0;
-    m_view_width = 0;
-    m_width = 0;
-
-    // connect to the signal manager
-    SignalManager *sig = &signal;
-    ASSERT(sig);
-    connect(sig, SIGNAL(sigTrackInserted(unsigned int, Track &)),
-            this, SLOT(slotTrackInserted(unsigned int, Track &)));
-    connect(sig, SIGNAL(sigTrackDeleted(unsigned int)),
-            this, SLOT(slotTrackDeleted(unsigned int)));
-    connect(sig, SIGNAL(sigSamplesDeleted(unsigned int, unsigned int,
-	unsigned int)),
-	this, SLOT(slotSamplesDeleted(unsigned int, unsigned int,
-	unsigned int)));
-    connect(sig, SIGNAL(sigSamplesInserted(unsigned int, unsigned int,
-	unsigned int)),
-	this, SLOT(slotSamplesInserted(unsigned int, unsigned int,
-	unsigned int)));
-    connect(sig, SIGNAL(sigSamplesModified(unsigned int, unsigned int,
-	unsigned int)),
-	this, SLOT(slotSamplesModified(unsigned int, unsigned int,
-	unsigned int)));
-
-    setBackgroundMode(NoBackground); // this avoids flicker :-)
+    // connect the timer for scrolling
     connect(&m_timer, SIGNAL(timeout()), this, SLOT(increase()));
+
+    // update the bitmap if the cache has changed
+    connect(&m_cache, SIGNAL(changed()), this, SLOT(refreshBitmap()));
+
+    // this avoids flicker :-)
+    setBackgroundMode(NoBackground);
 }
 
 //*****************************************************************************
@@ -93,7 +59,6 @@ OverViewWidget::~OverViewWidget()
 {
     m_timer.stop();
     if (m_pixmap) delete m_pixmap;
-    if (m_bitmap) delete m_bitmap;
 }
 
 //*****************************************************************************
@@ -274,22 +239,11 @@ void OverViewWidget::mouseMoveEvent( QMouseEvent *e)
     }
 }
 
-//*****************************************************************************
-void OverViewWidget::setOverView(QBitmap *overview)
-{
-    if (m_bitmap) delete m_bitmap;
-    m_bitmap = 0;
-
-    if (overview) m_bitmap = new QBitmap(*overview);
-
-    m_redraw = true;
-    repaint(false);
-}
-
 //*****************************************************************************
 void OverViewWidget::resizeEvent(QResizeEvent *)
 {
     setRange(m_view_offset, m_view_width, m_view_length);
+    refreshBitmap();
 }
 
 //*****************************************************************************
@@ -331,8 +285,9 @@ void OverViewWidget::setValue(unsigned int newval)
 }
 
 //*****************************************************************************
-void OverViewWidget::paintEvent (QPaintEvent *)
+void OverViewWidget::paintEvent(QPaintEvent *)
 {
+//    debug("OverViewWidget::paintEvent(QPaintEvent *)");
     QPainter p;
 
     // if pixmap has to be resized ...
@@ -343,14 +298,16 @@ void OverViewWidget::paintEvent (QPaintEvent *)
 	if (m_pixmap) delete m_pixmap;
 	m_pixmap = new QPixmap(size());
     }
+    if (!m_bitmap.width() || !m_bitmap.height()) refreshBitmap();
     ASSERT(m_pixmap);
     if (!m_pixmap) return;
 
+    // --- background of the widget ---
     p.begin(m_pixmap);
     m_pixmap->fill(BAR_BACKGROUND);
-    if (m_bitmap) {
+    if (m_bitmap.width() && m_bitmap.height()) {
 	QBrush brush;
-	brush.setPixmap(*m_bitmap);
+	brush.setPixmap(m_bitmap);
 	brush.setColor(BAR_FOREGROUND);
 	
 	p.setBrush(brush);
@@ -362,9 +319,9 @@ void OverViewWidget::paintEvent (QPaintEvent *)
 
     p.setBrush(SLIDER_BACKGROUND);
     p.drawRect(x, 0, m_slider_width, m_height);
-    if (m_bitmap) {
+    if (m_bitmap.width() && m_bitmap.height()) {
 	QBrush brush;
-	brush.setPixmap(*m_bitmap);
+	brush.setPixmap(m_bitmap);
 	brush.setColor(SLIDER_FOREGROUND);
 	p.setBrush(brush);
 	p.drawRect(x, 0, m_slider_width, m_height);
@@ -406,134 +363,13 @@ const QSize OverViewWidget::sizeHint()
 }
 
 //*****************************************************************************
-void OverViewWidget::scaleUp()
+void OverViewWidget::refreshBitmap()
 {
-    const unsigned int len = m_signal.length();
-    unsigned int new_scale = static_cast<unsigned int>(
-	rint(ceil(len/CACHE_SIZE)));
-//    debug("OverViewWidget::scaleUp(), new scale = %u", new_scale);
-
-    const unsigned int shrink = static_cast<unsigned int>(
-	rint(ceil(new_scale / m_scale)));
-//    debug("OverViewWidget::scaleUp(), shrink factor = %u", shrink);
-
-    for (unsigned int t=0; t < m_state.count(); ++t) {
-	unsigned int dst = 0;
-	unsigned int count = len / m_scale;
-	if (count > CACHE_SIZE) count = 0;
-	
-	char *smin = m_min.at(t)->data();
-	char *smax = m_max.at(t)->data();
-	CacheState *sstate = m_state.at(t)->data();
-	
-	char *dmin = smin;
-	char *dmax = smax;
-	CacheState *dstate = sstate;
-	
-	while (dst < count) {
-	    char min = +127;
-	    char max = -127;
-	    CacheState state = Unused;
-	    for (unsigned int i = 0; i < shrink; ++i) {
-		if (*smin < min) min = *smin;
-		if (*smax > max) max = *smax;
-		if (*sstate < state) state = *sstate;
-		++smin;
-		++smax;
-		++sstate;
-	    }
-	    *dmin = min;
-	    *dmax = max;
-	    *dstate = state;
-	    ++dmin;
-	    ++dmax;
-	    ++dstate;
-	    ++dst;
-	}
-	while (dst++ < CACHE_SIZE) {
-	    *dstate = Unused;
-	    dstate++;
-	}
-    }
-
-    m_scale = new_scale;
-}
-
-//*****************************************************************************
-void OverViewWidget::scaleDown()
-{
-}
-
-//*****************************************************************************
-void OverViewWidget::slotTrackInserted(unsigned int index, Track &)
-{
-//    debug("OverViewWidget::slotTrackInserted(%u,...)",index);
-    MutexGuard lock(m_lock);
-
-    QArray<CacheState> *state = new QArray<CacheState>(CACHE_SIZE);
-    QByteArray *min = new QByteArray(CACHE_SIZE);
-    QByteArray *max = new QByteArray(CACHE_SIZE);
-
-    if (!state || !min || !max) {
-	ASSERT(state);
-	ASSERT(min);
-	ASSERT(max);
-	if (state) delete state;
-	if (min) delete min;
-	if (max) delete max;
-	return;
-    }
-
-    min->fill(+127);
-    max->fill(-127);
-    state->fill(Unused);
-
-    m_min.insert(index, min);
-    m_max.insert(index, max);
-    m_state.insert(index, state);
-}
-
-//*****************************************************************************
-void OverViewWidget::slotTrackDeleted(unsigned int index)
-{
-//    debug("OverViewWidget::slotTrackDeleted(%u)",index);
-    MutexGuard lock(m_lock);
-
-    m_min.remove(index);
-    m_max.remove(index);
-    m_state.remove(index);
-
-    if (m_state.isEmpty()) m_scale = 0;
-}
-
-//*****************************************************************************
-void OverViewWidget::slotSamplesInserted(unsigned int track,
-    unsigned int offset, unsigned int length)
-{
-//    debug("OverViewWidget::slotSamplesInserted(%u,%u,%u)",track,offset,length);
-    MutexGuard lock(m_lock);
-
-    if ((m_signal.length() / m_scale) > CACHE_SIZE) scaleUp();
-
-}
-
-//*****************************************************************************
-void OverViewWidget::slotSamplesDeleted(unsigned int track,
-    unsigned int offset, unsigned int length)
-{
-//    debug("OverViewWidget::slotSamplesDeleted(%u,%u,%u)",track,offset,length);
-    MutexGuard lock(m_lock);
-
-    if ((m_signal.length() / m_scale) < (CACHE_SIZE/4)) scaleDown();
-}
-
-//*****************************************************************************
-void OverViewWidget::slotSamplesModified(unsigned int track,
-    unsigned int offset, unsigned int length)
-{
-//    debug("OverViewWidget::slotSamplesModified(%u,%u,%u)",track,offset,length);
-    MutexGuard lock(m_lock);
+    // let the bitmap be updated from the cache
+    m_bitmap = m_cache.getOverView(m_width, m_height);
 
+    // update the display
+    repaint(false);
 }
 
 //*****************************************************************************
