@@ -18,6 +18,7 @@
 #ifndef _TRACK_PIXMAP_H_
 #define _TRACK_PIXMAP_H_
 
+#include "config.h"
 #include <qarray.h>
 #include <qbitarray.h>
 #include <qcolor.h>
@@ -317,9 +318,12 @@ private:
     /** Color for interpolated samples */
     QColor m_color_interpolated;
 
-    /** Color for the zero line */
+    /** Color for the zero line, used areas */
     QColor m_color_zero;
 
+    /** Color of the zero line, unused areas */
+    QColor m_color_zero_unused;
+
 };
 
 #endif /* _TRACK_PIXMAP_H_ */
