@@ -19,6 +19,13 @@
 #include "libkwave/MultiTrackReader.h"
 #include "libkwave/SampleReader.h"
 
+//***************************************************************************
+MultiTrackReader::MultiTrackReader()
+    :QObject(), QVector<SampleReader>(), m_cancelled(false)
+{
+    setAutoDelete(true);
+}
+
 //***************************************************************************
 bool MultiTrackReader::eof() const
 {
@@ -32,5 +39,33 @@ bool MultiTrackReader::eof() const
     return false;
 }
 
+//***************************************************************************
+bool MultiTrackReader::insert(unsigned int track, const SampleReader *reader)
+{
+    if (reader) {
+        connect(reader, SIGNAL(proceeded()), this, SLOT(proceeded()));
+    }
+    return QVector<SampleReader>::insert(track, reader);
+}
+
+//***************************************************************************
+void MultiTrackReader::proceeded()
+{
+    unsigned int pos = 0;
+    unsigned int track;
+    const unsigned int tracks = count();
+    for (track=0; track < tracks; ++track) {
+	SampleReader *r = (*this)[track];
+	if (r) pos += (r->pos() - r->first());
+    }
+    emit progress(pos);
+}
+
+//***************************************************************************
+void MultiTrackReader::cancel()
+{
+    m_cancelled = true;
+}
+
 //***************************************************************************
 //***************************************************************************
