@@ -27,11 +27,13 @@
 // define this for using only slow Qt array functions
 #undef STRICTLY_QT
 
+#define BUFFER_SIZE (256*1024)
+
 //***************************************************************************
 SampleReader::SampleReader(Track &track, QList<Stripe> &stripes,
 	SampleLock *lock, unsigned int left, unsigned int right)
     :m_track(track), m_stripes(stripes), m_lock(lock),
-    m_src_position(left), m_first(left), m_last(right), m_buffer(16*1024),
+    m_src_position(left), m_first(left), m_last(right), m_buffer(BUFFER_SIZE),
     m_buffer_used(0), m_buffer_position(0)
 {
 }
@@ -48,6 +50,8 @@ void SampleReader::reset()
     m_src_position = m_first;
     m_buffer_used = 0;
     m_buffer_position = 0;
+
+    emit proceeded();
 }
 
 //***************************************************************************
@@ -85,6 +89,8 @@ void SampleReader::fillBuffer()
 	    rest -= cnt;
 	}
     }
+
+    emit proceeded();
 }
 
 //***************************************************************************
@@ -175,6 +181,31 @@ void SampleReader::skip(unsigned int count)
     }
 }
 
+//***************************************************************************
+void SampleReader::seek(unsigned int pos)
+{
+    const unsigned int current_pos = m_src_position +
+	m_buffer_position - m_buffer_used;
+
+    if (pos == current_pos) return; // nothing to do
+
+    if (pos < current_pos) {
+	// seek backwards
+	const unsigned int count = current_pos - pos;
+	if (count <= m_buffer_position) {
+	    // go back within the buffer
+	    m_buffer_position -= count;
+	} else {
+	    // skip out of the buffer
+	    m_src_position = pos;
+	    m_buffer_position = m_buffer_used = 0;
+	}
+    } else {
+	// seek forward
+	skip(pos - current_pos);
+    }
+}
+
 //***************************************************************************
 SampleReader &SampleReader::operator >> (sample_t &sample)
 {
