@@ -19,8 +19,10 @@
 #ifndef _SAMPLE_READER_H_
 #define _SAMPLE_READER_H_
 
+#include "config.h"
 #include <qarray.h>
 #include <qlist.h>
+#include <qobject.h>
 
 #include "libkwave/InsertMode.h"
 #include "libkwave/Sample.h"
@@ -29,8 +31,9 @@ class SampleLock;
 class Stripe;
 class Track;
 
-class SampleReader
+class SampleReader: public QObject
 {
+    Q_OBJECT
 public:
 
     /**
@@ -68,11 +71,12 @@ public:
     unsigned int read(QArray<sample_t> &buffer, unsigned int dstoff,
 	unsigned int length);
 
-    /**
-     * Skips a number of samples.
-     */
+    /** Skips a number of samples. */
     void skip(unsigned int count);
 
+    /** Seeks to a given position */
+    void seek(unsigned int pos);
+
     /**
      * Returns the current read position.
      */
@@ -102,6 +106,11 @@ public:
      */
     SampleReader& operator >> (QArray<sample_t> &sample);
 
+signals:
+
+    /** Emitted when the internal buffer is filled or the reader is closed */
+    void proceeded();
+
 protected:
 
     /** Fills the sample buffer */
