@@ -18,6 +18,7 @@
 #ifndef _SAMPLE_WRITER_H_
 #define _SAMPLE_WRITER_H_
 
+#include "config.h"
 #include <qarray.h>
 #include <qlist.h>
 #include <qobject.h>
@@ -121,6 +122,11 @@ public:
      */
     inline unsigned int last() { return m_last; };
 
+    /**
+     * Returns the current position
+     */
+    inline unsigned int position() { return m_position; };
+
 signals:
 
     /**
@@ -129,6 +135,9 @@ signals:
      */
     void sigSamplesWritten(unsigned int);
 
+    /** Emitted when the internal buffer is flushed or the writer is closed */
+    void proceeded();
+
 private:
 
     /** first sample */
