@@ -18,27 +18,27 @@
 #ifndef _ABOUT_KWAVE_DIALOG_H_
 #define _ABOUT_KWAVE_DIALOG_H_
 
-#include <kaboutapplication.h>
+#include "config.h"
+#include <qobject.h>
+#include "KwaveAboutDialogBase.uih.h"
+
+class QWidget;
 
 /**
  * @class AboutKwaveDialog
  * Dialog for Help/About
  */
-class AboutKwaveDialog : public KAboutApplication
+class AboutKwaveDialog : public KwaveAboutDialogBase
 {
     Q_OBJECT
 
 public:
-    /**
-     * Constructor
-     * @todo Change base class to KAboutDialog
-     * @todo Include logo or even better LogoWidget to the dialog front tab
-     * @todo Have tabs for developers, contributers, translators
-     */
+    /** Constructor */
     AboutKwaveDialog(QWidget *parent);
 
     /** destructor */
     virtual ~AboutKwaveDialog();
+
 };
 
 #endif  /* _ABOUT_KWAVE_DIALOG_H_ */
