@@ -31,6 +31,8 @@
 #include <qradiobutton.h>
 #include <qslider.h>
 #include <qstrlist.h>
+#include <qstring.h>
+#include <qstringlist.h>
 #include <qtooltip.h>
 
 #include "libkwave/WindowFunction.h"
@@ -392,11 +394,13 @@ void SonagramDialog::parameters(QStringList &list)
     list.append(param);
 
     // parameter #3: flag: track changes
-    param.setNum(m_rbColor ? (m_cbTrackChanges->isChecked() ? 1 : 0) : 0);
+    param.setNum((m_cbTrackChanges && m_cbTrackChanges->isChecked())
+        ? 1 : 0);
     list.append(param);
 
     // parameter #4: flag: follow selection
-    param.setNum(m_rbColor ? (m_cbFollowSelection->isChecked() ? 1 : 0) : 0);
+    param.setNum((m_cbFollowSelection && m_cbFollowSelection->isChecked())
+        ? 1 : 0);
     list.append(param);
 
 }
